#Tests interesting patterns of XML documents with pushbind

import unittest
import amara
from amara import binderytools

ROOT_GI_IN_CHILD = """\
<A>
  <A>One</A>
  <B>Two</B>
</A>
"""

MIXED_TOPLEVEL_CHILDREN_1 = """\
<A>
  <B id="1">One</B>
  <C id="2">Two</C>
  <B id="3">Three</B>
  <C id="3">Four</C>
</A>
"""

MIXED_TOPLEVEL_CHILDREN_2 = """\
<A>
  <B id="1">One</B>
  <C id="2">Two</C>
  <D id="3">Three</D>
  <B id="4">Four</B>
  <C id="5">Five</C>
  <D id="6">Six</D>
</A>
"""

MIXED_TOPLEVEL_CHILDREN_AND_GRANDKIDS = """\
<A>
  <B id="1">One</B>
  <C id="2">
    <B id="2.1">Two</B>
  </C>
  <B id="3">
    <C id="3.1">Three</C>
  </B>
  <C id="4">Three</C>
</A>
"""

ROOT_GI_IN_CHILD_GRANDCHILD = """\
<A>
  <A>
    <A>A One</A>
    <B>A Two</B>
  </A>
  <B>
    <A>B One</A>
    <B>B Two</B>
  </B>
</A>
"""

QNAME_CLASH_1 = """\
<a xmlns='http://example.com/ns1'>
  <a>One</a>
  <a xmlns='http://example.com/ns2'>Two</a>
  <b>Three</b>
</a>
"""

SPECIAL_GIS_1 = """<class class="1">
  <class class="a0"/>
  <class class="a1"/>
  <class class="a2"/>
</class>
"""

SPECIAL_GIS_2 = """<for for="1">
  <class class="a0"/>
  <class class="a1"/>
  <class class="a2"/>
</for>
"""

ATTRIBUTES_1 = """<r><x/><x a=\"1\"/><y/></r>"""


class TestInterestingXmlPatterns(unittest.TestCase):
    #def setUp(self):
    #    return

    def testToplevelKidsAndGrandkids1(self):
        elems = amara.pushbind(MIXED_TOPLEVEL_CHILDREN_AND_GRANDKIDS, u"/A/*")
        l1 = []
        l2 = []
        for e in elems:
            self.assertEqual(e.xml_parent, None)
            l1.append(e.nodeName)
            l2.append(e.id)
            self.assert_(isinstance(e.id, unicode))
            if e.id == u"2":
                self.assertEqual(len(list(e.B)), 1)
            if e.id == u"3":
                self.assertEqual(len(list(e.C)), 1)
        self.assertEqual(l1, [u"B", u"C", u"B", u"C"])
        self.assertEqual(l2, [u"1", u"2", u"3", u"4"])
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testToplevelKidsAndGrandkids2(self):
        elems = amara.pushbind(MIXED_TOPLEVEL_CHILDREN_AND_GRANDKIDS, u"B")
        l1 = []
        for e in elems:
            self.assertEqual(e.xml_parent, None)
            self.assertEqual(e.nodeName, u"B")
            l1.append(e.id)
            self.assert_(isinstance(e.id, unicode))
            if e.id == u"1":
                self.assertRaises(AttributeError, lambda x=None: e.C)
            if e.id == u"3":
                self.assertEqual(len(list(e.C)), 1)
                self.assertEqual(unicode(e.C), u"Three")
                self.assertEqual(e.C.id, u"3.1")
        self.assertEqual(l1, [u"1", u"2.1", u"3"])
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testQnameClash(self):
        PREFIXES = {u'ns1': u'http://example.com/ns1',
                    u'ns2': u'http://example.com/ns2'}
        elems = amara.pushbind(QNAME_CLASH_1, u"/*/*")
        elem = elems.next()
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns1')
        elem = elems.next()
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns2')
        elem = elems.next()
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns1')

        elems = amara.pushbind(QNAME_CLASH_1, u"/ns1:a/*",
                               prefixes=PREFIXES)
        elem = elems.next()
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns1')
        elem = elems.next()
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns2')
        elem = elems.next()
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns1')

        elems = amara.pushbind(QNAME_CLASH_1, u"/ns1:a",
                               prefixes=PREFIXES)
        elem = elems.next()
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns1')
        self.assertEqual(len(list(elem.a)), 1)
        self.assertEqual(len(list(elem.a_)), 1)
        self.assertEqual(len(list(elem.b)), 1)
        return

    def testSpecialGis1(self):
        elems = amara.pushbind(SPECIAL_GIS_1, u"/class/*")
        l = []
        for e in elems:
            self.assertEqual(e.nodeName, u"class")
            l.append(e.class_)
            self.assert_(isinstance(e.class_, unicode))
        self.assertEqual(l, [u"a0", u"a1", u"a2"])
        return

    def testSpecialGis2(self):
        elems = amara.pushbind(SPECIAL_GIS_2, u"/for/*")
        l = []
        for e in elems:
            self.assertEqual(e.nodeName, u"class")
            l.append(e.class_)
            self.assert_(isinstance(e.class_, unicode))
        self.assertEqual(l, [u"a0", u"a1", u"a2"])
        return

    def testAttributePatterns(self):
        patterns = {
            u'@a': [(2, u'a')],
            u'*/@a': [(2, u'a')],
            u'@a|r': [(1, u'r')],
            u'r|@a': [(1, u'r')],
            u'@a|x': [(1, u'x'), (1, u'x')],
            u'x|@a': [(1, u'x'), (1, u'x')],
            u'@a|y': [(2, u'a'), (1, u'y')],
            u'y|@a': [(2, u'a'), (1, u'y')],
            u'@a|r/x': [(1, u'x'), (1, u'x')],
            u'r/x|@a': [(1, u'x'), (1, u'x')],
            u'@a|r/y': [(2, u'a'), (1, u'y')],
            u'r/y|@a': [(2, u'a'), (1, u'y')],
            }
        for pat in patterns:
            nodes = amara.pushbind(ATTRIBUTES_1, pat)
            self.assertEqual(patterns[pat], [(n.nodeType, n.nodeName) for n in nodes])
        return


if __name__ == '__main__':
    unittest.main()





# -*- coding: iso-8859-1 -*-
#Required in Python 2.2, and must be the first import
from __future__ import generators
from xml.dom import Node

def doc_order_iter(node):
    """
    Iterates over each node in document order,
    yielding each in turn, starting with the given node
    node - the starting point
           (subtree rooted at node will be iterated over document order)
    """
    #Document order returns the current node,
    #then each of its children in turn
    yield node
    for child in node.childNodes:
        #Create a generator for each child,
        #Over which to iterate
        for cn in doc_order_iter(child):
            yield cn
    return


def doc_order_iter_filter(node, filter_func):
    """
    Iterates over each node in document order,
    applying the filter function to each in turn,
    starting with the given node, and yielding each node in
    cases where the filter function computes true
    node - the starting point
           (subtree rooted at node will be iterated over document order)
    filter_func - a callable object taking a node and returning
                  true or false
    """
    if filter_func(node):
        yield node
    for child in node.childNodes:
        for cn in doc_order_iter_filter(child, filter_func):
            yield cn
    return

def get_elements_by_tag_name(node, name):
    """
    Returns an iterator (in document order) over all the element nodes
    that are descendants of the given one, and have the given tag name.
y           node will be returned if it has the right name
    name - the node name to check
    """
    return doc_order_iter_filter(node, lambda n: n.nodeType == \
        Node.ELEMENT_NODE and n.nodeName == name)


def get_elements_by_tag_name_ns(node, ns, local):
    """
    Returns an iterator (in document order) over all the element nodes
    that are descendants of the given one, and have the given namespace
    and local name.
    node - the starting point (subtree rooted at node will be searched)
           node will be returned if it has the right ns and local name
    ns - the namespace to check
    local - the local name to check
    """
    return doc_order_iter_filter(node, lambda n: n.nodeType == \
        Node.ELEMENT_NODE and n.namespaceURI == ns and n.localName == local)


def get_first_element_by_tag_name_ns(node, ns, local):
    """
    Returns the first element in document order with the given namespace
    and local name
    node - the starting point (subtree rooted at node will be searched)
           node will be returned if it has the right ns and local name
    ns - the namespace to check
    local - the local name to check
    """
    return get_elements_by_tag_name_ns(node, ns, local).next()


def string_value(node):
    """
    Return the XPath string value of the given node
    This basically consists of one string comprising
    all the character data in the node and its descendants
    node - the starting point
    """
    text_nodes = doc_order_iter_filter(node, lambda n: n.nodeType == Node.TEXT_NODE)
    return u''.join([ n.data for n in text_nodes ])


def text_search(node, sought):
    """
    Return a list of descendant elements which contain a given substring
    in their text
    node - the starting point (subtree rooted at node will be searched)
    sought - the substring to find
    """
    return doc_order_iter_filter(node, lambda n: n.nodeType == Node.ELEMENT_NODE and [ 1 for cn in n.childNodes if cn.nodeType == Node.TEXT_NODE and cn.data.find(sought) != -1 ] )


def get_leaf_elements(node):
    """
    Return a list of elements that have no element children
    node - the starting point (subtree rooted at node will be searched)
    """
    return doc_order_iter_filter(node, lambda n: n.nodeType == Node.ELEMENT_NODE and not [ 1 for cn in n.childNodes if cn.nodeType == Node.ELEMENT_NODE ])


def rename_element(elem, new_ns, new_qname):
    new = elem.rootNode.createElementNS(new_ns, new_qname)
    for child in elem.childNodes:
        elem.removeChild(child)
        new.appendChild(child)
    elem.parentNode.replaceChild(new, elem)
    return new

#
# The following functions do not use the generator/iterator framework
#

#Mapping from node type to XPath node test function name
OTHER_NODES = {
    Node.TEXT_NODE: 'text',
    Node.COMMENT_NODE: 'comment',
    Node.PROCESSING_INSTRUCTION_NODE: 'processing-instruction'
    }

from Ft.Xml.Domlette import GetAllNs, SeekNss
DEFAULT_GENERATED_PREFIX = u"amara.4suite.org.ns%i"

def abs_path(node, prefixes=None):
    #based on code developed by Florian Bosch on XML-SIG
    #http://mail.python.org/pipermail/xml-sig/2004-August/010423.html
    #Significantly enhanced to use Unicode properly, support more
    #node types, use safer node type tests, etc.
    """
    Return an XPath expression that provides a unique path to
    the given node (supports elements, attributes, root nodes,
    text nodes, comments and PIs) within a document

    prefixes - optional hint dictionary from prefix to namespace
               if the document uses the default namespace, this function
               might need to come up with a prefix for the QName for certain
               nodes.  You can help control such prefix selection by passing in
               this dictionary.  Prefixes found in this dictionary will
               be used to reconcile default namespace usage
               WARNING: this value may be mutated by this function--key/value
               pairs might be added to the dictionary
    """
    if node.nodeType == Node.ELEMENT_NODE:
        count = 1
        #Count previous siblings with same node name
        previous = node.previousSibling
        while previous:
            if ((previous.namespaceURI, previous.localName)
                == (node.namespaceURI, node.localName)):
                count += 1
            previous = previous.previousSibling
        qname = node.nodeName
        if node.namespaceURI and not node.prefix:
            if prefixes is None:
                prefixes = GetAllNs(node)
            #nicer code, but maybe slower than iterating items()
            nss = dict([(n,p) for (p,n) in prefixes.items()])
            #must provide a prefix for XPath
            prefix = nss.get(node.namespaceURI)
            if not prefix:
                prefix_count = 1
                prefix = DEFAULT_GENERATED_PREFIX%count
                while prefix in prefixes:
                    prefix_count += 1
                    prefix = DEFAULT_GENERATED_PREFIX%count
            prefixes[prefix] = node.namespaceURI
            qname = prefix + u':' + node.localName
        step = u'%s[%i]' % (qname, count)
        ancestor = node.parentNode
    elif node.nodeType == Node.ATTRIBUTE_NODE:
        step = u'@%s' % (node.nodeName)
        ancestor = node.ownerElement
    elif node.nodeType in OTHER_NODES:
        #Text nodes, comments and PIs
        count = 1
        #Count previous siblings of the same node type
        previous = node.previousSibling
        while previous:
            if previous.nodeType == node.nodeType: count += 1
            previous = previous.previousSibling
        test_func = OTHER_NODES[node.nodeType]
        step = u'%s()[%i]' % (test_func, count)
        ancestor = node.parentNode
    elif not node.parentNode:
        #Root node
        step = u''
        ancestor = node
    else:
        raise TypeError('Unsupported node type for abs_path')
    if ancestor.parentNode:
        return abs_path(ancestor, prefixes) + u'/' + step
    else:
        return u'/' + step


import xml.sax
from Ft.Xml import Sax
from amara.saxtools import sax2dom_chunker
from Ft.Xml import InputSource
from Ft.Lib import Uri, Uuid
from Ft.Xml.Lib.XmlString import IsXml

def pushdom(source, xpatterns, prefixes=None, validate=False):
    parser = Sax.CreateParser()
    if validate:
        parser.setFeature(xml.sax.handler.feature_validation, True)
    else:
        parser.setFeature(xml.sax.handler.feature_external_pes, False)
    parser.setFeature(Sax.FEATURE_GENERATOR, True)

    def handle_chunk(docfrag):
        parser.setProperty(Sax.PROPERTY_YIELD_RESULT, docfrag)

    handler = sax2dom_chunker(xpatterns=xpatterns,
                              nss=prefixes,
                              chunk_consumer=handle_chunk)
    parser.setContentHandler(handler)
    if isinstance(source, InputSource.InputSource):
        pass
    elif hasattr(source, 'read'):
        #Create dummy Uri to use as base
        dummy_uri = 'urn:uuid:'+Uuid.UuidAsString(Uuid.GenerateUuid())
        source = InputSource.DefaultFactory.fromStream(source, dummy_uri)
    elif IsXml(source):
        #Create dummy Uri to use as base
        dummy_uri = 'urn:uuid:'+Uuid.UuidAsString(Uuid.GenerateUuid())
        source = InputSource.DefaultFactory.fromString(source, dummy_uri)
    elif Uri.IsAbsolute(source): #or not os.path.isfile(source):
        source = InputSource.DefaultFactory.fromUri(source)
    else:
        source = InputSource.DefaultFactory.fromUri(Uri.OsPathToUri(source))

    return parser.parse(source)


#def insert_xml(node, position, text):
#    """
#    Parse some text and insert it as a child of an element
#    """




#Tests bindery rules

import unittest
import amara

LABELS = """\
<?xml version="1.0" encoding="iso-8859-1"?>
<labels>
  <label id='ep' added="2003-06-10">
    <name>Ezra Pound</name>
    <address>
      <street>45 Usura Place</street>
      <city>Hailey</city>
      <province>ID</province>
    </address>
  </label>
  <label id='tse' added="2003-06-20">
    <name>Thomas Eliot</name>
    <address>
      <street>3 Prufrock Lane</street>
      <city>Stamford</city>
      <province>CT</province>
    </address>
  </label>
  <label id="lh" added="2004-11-01">
    <name>Langston Hughes</name>
    <address>
      <street>10 Bridge Tunnel</street>
      <city>Harlem</city>
      <province>NY</province>
    </address>
  </label>
  <label id="co" added="2004-11-15">
    <name>Christopher Okigbo</name>
    <address>
      <street>7 Heaven's Gate</street>
      <city>Idoto</city>
      <province>Anambra</province>
    </address>
  </label>
</labels>
"""

XBEL = """\
<?xml version="1.0" encoding="utf-8"?>
<!--
<!DOCTYPE xbel PUBLIC "+//IDN python.org//DTD XML Bookmark Exchange Language 1.0//EN//XML" "http://www.python.org/topics/xml/dtds/xbel-1.0.dtd">
-->
<xbel>
  <title>Some 4Suite Bookmarks</title>
  <!-- No top level bookmars -->
  <folder>
    <title>Main links</title>
    <bookmark href="http://4suite.org/">
      <title>4Suite home page</title>
    </bookmark>
    <bookmark href="http://foursuite.sourceforge.net/">
      <title>4Suite on SourceForge</title>
    </bookmark>
    <bookmark href="http://uche.ogbuji.net/tech/4Suite/">
      <title>Uche's 4Suite page</title>
    </bookmark>
    <bookmark href="http://twistedmatrix.com/users/jh.twistd/xml-sig/moin.cgi/FtSuiteWiki">
      <title>4Suite Wiki</title>
    </bookmark>
  </folder>
  <folder>
    <title>Articles</title>
    <bookmark href="http://www.xml.com/pub/a/2002/10/16/py-xml.html">
      <title>Python &amp; XML: A Tour of 4Suite</title>
    </bookmark>
    <folder>
      <title>Free registration required</title>
      <bookmark href="http://www-105.ibm.com/developerworks/education.nsf/xml-onlinecourse-bytitle/BE1A7E60838F9F7686256AF400523C58">
        <title>Python and XML development using 4Suite, Part 2: 4XPath and 4XSLT </title>
      </bookmark>
      <bookmark href="http://www-105.ibm.com/developerworks/education.nsf/xml-onlinecourse-bytitle/8A1EA5A2CF4621C386256BBB006F4CEC">
        <title>Python and XML development using 4Suite, Part 3: 4RDF</title>
      </bookmark>
      <bookmark href="http://www-105.ibm.com/developerworks/education.nsf/xml-onlinecourse-bytitle/CD1A8865668D9A2E86256C910062D445?OpenDocument">
        <title>Develop Python/XML with 4Suite, Part 5: The Repository Features</title>
      </bookmark>
    </folder>
  </folder>
  <folder>
    <title>Mailing lists</title>
    <bookmark href="http://lists.fourthought.com/mailman/listinfo/4suite">
      <title>4Suite Mailing List</title>
    </bookmark>
    <bookmark href="http://lists.fourthought.com/mailman/listinfo/4suite-ann">
      <title>4Suite Announcements Mailing List</title>
    </bookmark>
    <bookmark href="http://lists.fourthought.com/mailman/listinfo/4suite-dev">
      <title>4Suite Developer Mailing List</title>
    </bookmark>
  </folder>
</xbel>
"""

RSS = """\
<?xml version="1.0"?>
<rdf:RDF 
  xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" 
  xmlns:dc="http://purl.org/dc/elements/1.1/"
  xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"
  xmlns="http://purl.org/rss/1.0/"
> 

  <channel rdf:about="http://meerkat.oreillynet.com/">
    <title>Meerkat: An Open Wire Service</title> 
    <link>http://meerkat.oreillynet.com/</link> 
    <description>
      This Meerkat flavour, rss10, conforms to the current status of
      RSS 1.0 development.  Please note that it may change at any time.
    </description>
    <sy:updatePeriod>hourly</sy:updatePeriod>
    <sy:updateFrequency>2</sy:updateFrequency>
    <sy:updateBase>2000-01-01T12:00+00:00</sy:updateBase>

    <image rdf:resource="http://meerkat.oreillynet.com/icons/meerkat-powered.jpg" />

    <items>
      <rdf:Seq>
        <rdf:li rdf:resource="http://www.oreillynet.com/cs/weblog/view/wlg/536" />
        <rdf:li rdf:resource="http://www.oreillynet.com/cs/weblog/view/wlg/534" />
      </rdf:Seq>
    </items>
  
    <textinput rdf:resource="http://meerkat.oreillynet.com/#search" />

  </channel>

  <image rdf:about="http://meerkat.oreillynet.com/icons/meerkat-powered.jpg">
    <title>Meerkat Powered!</title>
    <url>http://meerkat.oreillynet.com/icons/meerkat-powered.jpg</url>
    <link>http://meerkat.oreillynet.com</link>
  </image>

  <item rdf:about="http://www.oreillynet.com/cs/weblog/view/wlg/536">
    <title>Learn Java in 14 Lines! by Frank Willison</title>
    <link>http://www.oreillynet.com/cs/weblog/view/wlg/536</link>
    <description> In Memory of Frank Willison, O&#039;Reilly Editor in Chief, we find his take on learning Java with a sample tutorial sonnet, &amp;quot;Alas! I Married a Java Applet!,&amp;quot; from his Learn Java in 14 Lines!. </description>
    <dc:source>O&#039;Reilly Network Weblogs</dc:source>
    <dc:creator>Steve Anglin</dc:creator>
    <dc:subject>Java</dc:subject>
    <dc:publisher>O'Reilly and Associates</dc:publisher>
    <dc:date>2001-08-02 10:42:42</dc:date>
    <dc:format>text/html</dc:format>
    <dc:language>en-us</dc:language>
    <dc:rights>Copyright 2001, O'Reilly and Associates</dc:rights>
  </item>

  <item rdf:about="http://www.oreillynet.com/cs/weblog/view/wlg/534">
    <title>IBM joins push to construct next-generation internet</title>
    <link>http://www.oreillynet.com/cs/weblog/view/wlg/534</link>
    <description> The movement to build a new-generation internet through a global computer &amp;quot;grid&amp;quot; will receive a big boost on Thursday when International Business Machines, the world&#039;s largest computing company, commits itself to the new technology. </description>
    <dc:source>O&#039;Reilly Network Weblogs</dc:source>
    <dc:creator>John Scott</dc:creator>
    <dc:subject>P2P</dc:subject>
    <dc:publisher>O'Reilly and Associates</dc:publisher>
    <dc:date>2001-08-02 07:20:52</dc:date>
    <dc:format>text/html</dc:format>
    <dc:language>en-us</dc:language>
    <dc:rights>Copyright 2001, O'Reilly and Associates</dc:rights>
  </item>

</rdf:RDF>
"""

class TestXPath(unittest.TestCase):
    #def setUp(self):
    #    return

    def testAttrAxis1(self):
        doc = amara.parse(LABELS)
        self.assert_(hasattr(doc.labels.label, "xml_attributes"))
        self.assertEqual(len(list(doc.labels)), 1)
        self.assertEqual(len(list(doc.labels.label)), 4)
        self.assertEqual(len(list(doc.labels.label.name)), 1)
        self.assertEqual(len(list(doc.labels.label.address)), 1)
        self.assertEqual(len(list(doc.labels.label.address.city)), 1)
        self.assertEqual(len(list(doc.labels.label.address.street)), 1)
        self.assertEqual(len(list(doc.labels.label.address.province)), 1)
        self.assertEqual(doc.xml_xpath(u"count(//@id)"), 4.0)
        self.assertEqual(doc.xml_xpath(u"count(//@*)"), 8.0)
        self.assertEqual(doc.xml_xpath(u"string(/labels/label[@id='ep']/name)"), u"Ezra Pound")
        return

    def testAttrAxis2(self):
        XML = '<html><test href="bar" /><test href="foo" /></html>'
        doc = amara.parse(XML)
        self.assertEqual(doc.xml_xpath(u"count(//@href)"), 2.0)
        self.assert_([ a.value for a in doc.xml_xpath(u"//@href")] in [[u"foo", u"bar"], [u"bar", u"foo"]])
        self.assertEqual(doc.xml_xpath(u"count(//@href|//@bogus)"), 2.0)
        self.assert_([ a.value for a in doc.xml_xpath(u"//@href|//@bogus")] in [[u"foo", u"bar"], [u"bar", u"foo"]])
        return

    def testAttrAxis3(self):
        #Reported by Luis Miguel Morillas
        #http://lists.fourthought.com/pipermail/4suite/2006-November/008136.html
        XML = '<a><!-- example --><b>blablabla</b></a>'
        doc = amara.parse(XML)
        self.assertEqual(len(doc.xml_xpath(u"//*")), 2)
        self.assertEqual(len(doc.xml_xpath(u"//@*")), 0)
        return

    def testAttrAxis4(self):
        XML = '<a><?target data?><b>blablabla</b></a>'
        doc = amara.parse(XML)
        self.assertEqual(len(doc.xml_xpath(u"//*")), 2)
        self.assertEqual(len(doc.xml_xpath(u"//@*")), 0)
        return

    def testAttrAxisNs(self):
        RSS10_NSS = {
            u'rdf': u'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
            u'dc': u'http://purl.org/dc/elements/1.1/',
            u'rss': u'http://purl.org/rss/1.0/',
            }
        doc = amara.parse(RSS, prefixes=RSS10_NSS)
        self.assert_(not hasattr(doc.RDF, "xml_attributes"))
        self.assert_(hasattr(doc.RDF.channel, "xml_attributes"))
        self.assertEqual(len(list(doc.RDF)), 1)
        self.assertEqual(doc.xml_xpath(u"count(//@rdf:about)"), 4.0)
        self.assertEqual(doc.xml_xpath(u"count(//rss:item/@rdf:about)"), 2.0)
        self.assertEqual(doc.xml_xpath(u"count(//@rdf:resource)"), 4.0)
        self.assertEqual(doc.xml_xpath(u"count(//rdf:li/@rdf:resource)"), 2.0)
        self.assertEqual(
            doc.xml_xpath(u"string(//rss:item[@rdf:about='http://www.oreillynet.com/cs/weblog/view/wlg/534']/rss:title)"), u"IBM joins push to construct next-generation internet"
            )
        return

if __name__ == '__main__':
    unittest.main()




#Tests mutation

import unittest
import cStringIO
import amara
from Ft.Xml.Domlette import PrettyPrint
from Ft.Xml.Xslt.XmlWriter import XmlWriter
from Ft.Xml.Xslt.OutputParameters import OutputParameters
from xml.dom import Node


XMLDECL = '<?xml version="1.0" encoding="UTF-8"?>\n'


class TestBasicMods(unittest.TestCase):
    #def setUp(self):
    #    return

    def testBasicCreateDoc1(self):
        EXPECTED = "<A/>"
        doc = amara.create_document(u"A")
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 0)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testBasicCreateDoc2(self):
        EXPECTED = '<A xmlns:ns="urn:bogus" ns:a="b"/>'
        doc = amara.create_document(
            u"A", attributes={(u"ns:a", u"urn:bogus"): u"b"})
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 0)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testBasicCreateDoc3(self):
        EXPECTED = '<A a="b"/>'
        #Namespace-free attr creation, abbreviated
        doc = amara.create_document(
            u"A", attributes={u"a": u"b"})
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 0)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #Namespace-free attr creation, full
        doc = amara.create_document(
            u"A", attributes={(u"a", None): u"b"})
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 0)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testBasicCreateDoc4(self):
        doc = amara.create_document(u"A")
        e1 = doc.xml_create_element(u"B", content=u"One",
                             attributes={(u"ns:a", u"urn:bogus"): u"b"})
        doc.A.xml_append(e1)
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 1)
        EXPECTED = '<A><B xmlns:ns="urn:bogus" ns:a="b">One</B></A>'
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testBasicCreateDoc4(self):
        EXPECTED = '<A><B a="b">One</B></A>'
        #Namespace-free attr creation, abbreviated
        doc = amara.create_document(u"A")
        e1 = doc.xml_create_element(u"B", content=u"One",
                             attributes={u"a": u"b"})
        doc.A.xml_append(e1)
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 1)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #Namespace-free attr creation, full
        doc = amara.create_document(u"A")
        e1 = doc.xml_create_element(u"B", content=u"One",
                             attributes={(u"a", None): u"b"})
        doc.A.xml_append(e1)
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 1)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testBasicCreateDoc5(self):
        EXPECTED = '<A a="b"/>'
        doc = amara.create_document(u"A")
        doc.A.a = u"b"
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 0)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testBasicCreateDoc6(self):
        EXPECTED = '<A xmlns:ns="urn:bogus" ns:a="b"/>'
        doc = amara.create_document(u"A")
        doc.A.xml_set_attribute((u"ns:a", u"urn:bogus"), u"b")
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 0)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testCreateDoc2(self):
        doc = amara.create_document(u"A")
        e1 = doc.xml_create_element(u"A", content=u"One")
        e2 = doc.xml_create_element(u"B", content=u"Two")
        doc.A.xml_append(e1)
        doc.A.xml_append(e2)
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(list(doc.A.A)), 1)
        self.assertEqual(len(list(doc.A.B)), 1)
        self.assertEqual(unicode(doc.A.A), u"One")
        self.assertEqual(unicode(doc.A.B), u"Two")
        EXPECTED = "<A><A>One</A><B>Two</B></A>"
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        EXPECTED = "<A>\n  <A>One</A>\n  <B>Two</B>\n</A>"
        op = OutputParameters()
        op.indent = u"yes"
        stream = cStringIO.StringIO()
        w = XmlWriter(op, stream)
        doc.xml(writer=w)
        self.assertEqual(stream.getvalue(), XMLDECL+EXPECTED)
        #PrettyPrint(doc)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testCreateDocNs1(self):
        EXPECTED = '<A xmlns="urn:bogus"/>'
        doc = amara.create_document(u"A", u"urn:bogus")
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 0)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testTemplate1(self):
        EXPECTED = '<A><B a="b">One</B></A>'
        doc = amara.create_document(u"A")
        doc.A.xml_append_fragment('<B a="b">One</B>')
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 1)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testTemplate2(self):
        EXPECTED = '<A><B xmlns:ns="urn:bogus" ns:a="b">One</B></A>'
        doc = amara.create_document(u"A")
        doc.A.xml_append_fragment('<B xmlns:ns="urn:bogus" ns:a="b">One</B>')
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 1)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testTemplate3(self):
        EXPECTED = '<A xmlns:ns="urn:bogus" ns:a="b"><B>One</B></A>'
        doc = amara.create_document(u"A", attributes={(u'ns:a', u"urn:bogus"): u'b'})
        doc.A.xml_append_fragment('<B>One</B>')
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 1)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testTemplate4(self):
        EXPECTED = '<A><B xmlns:ns="urn:bogus" ns:a="b">One</B></A>'
        doc = amara.create_document(u"A")
        doc.A.xml_append_fragment('<B xmlns:ns="urn:bogus" ns:a="b">One</B>')
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 1)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testTemplate5(self):
        EXPECTED = u'<A><B>\u2203</B></A>'.encode('utf-8')
        doc = amara.create_document(u'A')
        doc.A.xml_append_fragment(u'<B>\u2203</B>'.encode('utf-8'))
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 1)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testTemplate6(self):
        EXPECTED = u'<A><B>\u00AB\u00BB</B></A>'.encode('utf-8')
        doc = amara.create_document(u'A')
        doc.A.xml_append_fragment(u'<B>\u00AB\u00BB</B>'.encode('latin-1'), 'latin-1')
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 1)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return


    def testCreateDocType1(self):
        EXPECTED = '<!DOCTYPE xsa PUBLIC "-//LM Garshol//DTD XML Software Autoupdate 1.0//EN//XML" "http://www.garshol.priv.no/download/xsa/xsa.dtd">\n<xsa/>'
        doc = amara.create_document(
            u"xsa",
            pubid=u"-//LM Garshol//DTD XML Software Autoupdate 1.0//EN//XML",
            sysid=u"http://www.garshol.priv.no/download/xsa/xsa.dtd"
            )
        self.assertEqual(len(list(doc.xsa)), 1)
        self.assertEqual(len(doc.xsa.xml_children), 0)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        #PrettyPrint(doc)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testCreateDocType2(self):
        EXPECTED = '<!DOCTYPE xsa PUBLIC "-//LM Garshol//DTD XML Software Autoupdate 1.0//EN//XML" "http://www.garshol.priv.no/download/xsa/xsa.dtd">\n<xsa/>'
        doc = amara.create_document(
            u"xsa",
            pubid=u"-//LM Garshol//DTD XML Software Autoupdate 1.0//EN//XML",
            sysid=u"http://www.garshol.priv.no/download/xsa/xsa.dtd"
            )
        self.assertEqual(len(list(doc.xsa)), 1)
        self.assertEqual(len(doc.xsa.xml_children), 0)
        self.assertEqual(doc.xml(indent=u'yes'), XMLDECL+EXPECTED)
        #PrettyPrint(doc)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testCreateDocType4(self):
        EXPECTED = '<!DOCTYPE xsa PUBLIC "-//LM Garshol//DTD XML Software Autoupdate 1.0//EN//XML" "http://www.garshol.priv.no/download/xsa/xsa.dtd">\n<xsa/>'
        op = OutputParameters()
        op.indent = u'yes'
        op.doctypeSystem = u"http://www.garshol.priv.no/download/xsa/xsa.dtd"
        op.doctypePublic = u"-//LM Garshol//DTD XML Software Autoupdate 1.0//EN//XML"
        stream = cStringIO.StringIO()
        w = XmlWriter(op, stream)
        doc = amara.create_document(u"xsa")
        self.assertEqual(len(list(doc.xsa)), 1)
        self.assertEqual(len(doc.xsa.xml_children), 0)
        doc.xml(writer=w)
        self.assertEqual(stream.getvalue(), XMLDECL+EXPECTED)
        #PrettyPrint(doc)
        #self.assert_(isinstance(doc.xbel.title, unicode))
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testReplace(self):
        EXPECTED = '<A><B id="1">One</B><B id="2">Two</B></A>'
        DOC = EXPECTED
        doc = amara.parse(DOC)
        del doc.A.B[1]
        e2 = doc.xml_create_element(u"B", content=u"Two",
                             attributes={u"id": u"2"})
        doc.A.xml_append(e2)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def XXXtestRepeatEdits1(self):
        print "testRepeatEdits1"
        #EXPECTED = '<A><B a="b">One</B></A>'
        doc = amara.create_document(u"A")
        e1 = doc.xml_create_element(u"B", content=u"One",
                             attributes={u"a": u"b"})
        doc.A.xml_append(e1)
        self.assertEqual(len(list(doc.A)), 1)
        self.assertEqual(len(doc.A.xml_children), 1)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetChildElement1(self):
        DOC = "<a><b>spam</b></a>"
        EXPECTED = '<a><b>eggs</b></a>'
        doc = amara.parse(DOC)
        doc.a.b = u"eggs"
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetChildElement2(self):
        DOC = "<a><b>spam</b></a>"
        EXPECTED = '<a><b>eggs</b></a>'
        doc = amara.parse(DOC)
        doc.a.b[0] = u"eggs"
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetChildElement3(self):
        DOC = "<a><b>spam</b><b>spam</b></a>"
        EXPECTED = '<a><b>eggs</b><b>spam</b></a>'
        doc = amara.parse(DOC)
        doc.a.b = u"eggs"
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetChildElement4(self):
        DOC = "<a><b>spam</b><b>spam</b></a>"
        EXPECTED = '<a><b>eggs</b><b>spam</b></a>'
        doc = amara.parse(DOC)
        doc.a.b[0] = u"eggs"
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetChildElement5(self):
        DOC = "<a><b>spam</b><b>spam</b></a>"
        EXPECTED = '<a><b>spam</b><b>eggs</b></a>'
        doc = amara.parse(DOC)
        doc.a.b[1] = u"eggs"
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetChildElement6(self):
        DOC = "<a><b>spam</b><b>spam</b></a>"
        doc = amara.parse(DOC)
        def edit():
            doc.a.b[2] = u"eggs"
        self.assertRaises(IndexError, edit)
        return

    def testDelChildElement1(self):
        DOC = "<a><b>spam</b><b>eggs</b></a>"
        EXPECTED = '<a><b>eggs</b></a>'
        doc = amara.parse(DOC)
        del doc.a.b
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testDelChildElement2(self):
        DOC = "<a><b>spam</b><b>eggs</b></a>"
        EXPECTED = '<a><b>eggs</b></a>'
        doc = amara.parse(DOC)
        del doc.a.b[0]
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testDelChildElement3(self):
        DOC = "<a><b>spam</b><b>eggs</b></a>"
        EXPECTED = '<a><b>spam</b></a>'
        doc = amara.parse(DOC)
        del doc.a.b[1]
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testDelChildElement4(self):
        DOC = "<a><b>spam</b><b>spam</b></a>"
        doc = amara.parse(DOC)
        def edit():
            del doc.a.b[2]
        self.assertRaises(IndexError, edit)
        return

    def testDelChildElement5(self):
        DOC = "<a><b>spam</b><b>eggs</b></a>"
        EXPECTED = '<a><b>eggs</b></a>'
        doc = amara.parse(DOC)
        del doc.a[u'b']
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testDelChildElement6(self):
        DOC = "<a><b>spam</b><b>eggs</b></a>"
        EXPECTED = '<a><b>eggs</b></a>'
        doc = amara.parse(DOC)
        del doc.a[u'b'][0]
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testDelChildElement7(self):
        DOC = "<a><b>spam</b><b>eggs</b></a>"
        EXPECTED = '<a><b>spam</b></a>'
        doc = amara.parse(DOC)
        del doc.a[u'b'][1]
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testDelChildElementWithClash1(self):
        DOC = '<a-1 b-1=""><b-1/></a-1>'
        EXPECTED = '<a-1 b-1=""/>'
        doc = amara.parse(DOC)
        E = Node.ELEMENT_NODE
        del doc[E, None, u'a-1'][E, None, u'b-1']
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testDelAttributeWithClash1(self):
        DOC = '<a-1 b-1=""><b-1/></a-1>'
        EXPECTED = '<a-1><b-1/></a-1>'
        doc = amara.parse(DOC)
        E = Node.ELEMENT_NODE
        A = Node.ATTRIBUTE_NODE
        del doc[E, None, u'a-1'][A, None, u'b-1']
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testDelAttribute1(self):
        DOC = '<a b="spam"><b>spam</b></a>'
        EXPECTED = '<a><b>spam</b></a>'
        doc = amara.parse(DOC)
        del doc.a.b
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetAttribute1(self):
        DOC = '<a b="spam"></a>'
        EXPECTED = '<a b="eggs"/>'
        doc = amara.parse(DOC)
        doc.a.b = u"eggs"
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetAttribute2(self):
        DOC = '<a b="spam"><b>spam</b></a>'
        EXPECTED = '<a b="eggs"><b>spam</b></a>'
        doc = amara.parse(DOC)
        doc.a.b = u"eggs"
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetAttribute3(self):
        from xml.dom import Node
        DOC = '<a b="spam"><b>spam</b></a>'
        EXPECTED = '<a b="eggs"><b>spam</b></a>'
        doc = amara.parse(DOC)
        doc.a[Node.ATTRIBUTE_NODE, None, u'b'] = u'eggs'
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetAttribute4(self):
        DOC = '<a><b>spam</b></a>'
        EXPECTED = '<a><b xml:lang="en">spam</b></a>'
        doc = amara.parse(DOC)
        doc.a.b.xml_set_attribute((u"xml:lang"), u"en")
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetAttribute5(self):
        DOC = '<a><b>spam</b></a>'
        EXPECTED = '<a xmlns:ns="urn:bogus" ns:foo="bar"><b>spam</b></a>'
        doc = amara.parse(DOC)
        doc.a.xml_set_attribute((u"ns:foo", u"urn:bogus"), u"bar")
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetAttribute6(self):
        DOC = '<a><b>spam</b></a>'
        EXPECTED = '<a xmlns:ns="urn:bogus" ns:foo="bar"><b>spam</b></a>'
        doc = amara.parse(DOC, prefixes={u'ns': u'urn:bogus'})
        doc.a.xml_set_attribute((u"foo", u"urn:bogus"), u"bar")
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testSetAttribute7(self):
        DOC = '<a><b>spam</b></a>'
        EXPECTED = '<a foo="bar"><b>spam</b></a>'
        doc = amara.parse(DOC)
        doc.a.xml_set_attribute(u"foo", u"bar")
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testInsertAfter1(self):
        DOC = "<a><b>spam</b></a>"
        EXPECTED = '<a><b>spam</b><b>eggs</b></a>'
        doc = amara.parse(DOC)
        doc.a.xml_insert_after(doc.a.b, doc.a.xml_create_element(u'b', content=u"eggs"))
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testInsertAfter2(self):
        DOC = "<a><b>spam</b></a>"
        EXPECTED = '<a><b>spam</b><c>eggs</c></a>'
        doc = amara.parse(DOC)
        doc.a.xml_insert_after(doc.a.b, doc.a.xml_create_element(u'c', content=u"eggs"))
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testInsertAfter3(self):
        DOC = "<a><b>spam</b><c>ham</c><c>pork</c></a>"
        EXPECTED = "<a><b>spam</b><c>eggs</c><c>ham</c><c>pork</c></a>"
        doc = amara.parse(DOC)
        doc.a.xml_insert_after(doc.a.b, doc.a.xml_create_element(u'c',
                                                     content=u'eggs'))
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return

    def testInsertBefore1(self):
        DOC = "<a><b>eggs</b></a>"
        EXPECTED = '<a><b>spam</b><b>eggs</b></a>'
        doc = amara.parse(DOC)
        doc.a.xml_insert_before(doc.a.b, doc.a.xml_create_element(u'b', content=u"spam"))
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        return


MONTY = """\
<?xml version="1.0" encoding="utf-8"?>
<monty>
  <python spam="eggs">
    What do you mean "bleh"
  </python>
  <python ministry="abuse">
    But I was looking for argument
  </python>
</monty>
"""


class TestTransforms(unittest.TestCase):
    def testDeepCopy(self):
        #FIXME really goes in manual.py, since it's based on a manual example
        EXPECTED1 = '<python spam="eggs">\n    What do you mean "bleh"\n  </python>'
        EXPECTED2 = '<python spam="abcd">\n    What do you mean "bleh"\n  </python>'
        import copy
        doc = amara.parse(MONTY)
        doc2 = copy.deepcopy(doc)
        doc2.monty.python.spam = u"abcd"

        self.assertEqual(doc.monty.python.xml(), EXPECTED1)
        self.assertEqual(doc2.monty.python.xml(), EXPECTED2)
        return


if __name__ == '__main__':
    unittest.main()




"""
Demonstrate sibling grouping, a common PITA problem.
Specifically inspired by:
http://mail.python.org/pipermail/xml-sig/2005-March/011028.html
"""
import amara

XML="""\
<doc>
  <a id="1"/>
  <b id="1.1"/>
  <c id="1.2"/>
  <a id="2"/>
  <b id="2.1"/>
  <c id="2.2"/>
  <a id="3"/>
  <b id="3.1"/>
  <c id="3.2"/>
</doc>
"""

top = amara.create_document(u"doc")

container = None
for e in amara.pushbind(XML, u'/doc/*'):
    if e.nodeName == u"a":
        container = e
        top.doc.xml_append(e)
    else:
        container.xml_append(e)

print top.xml(indent=u"yes")




#
# The command line code executes a simple battery of tests
#
from amara import domtools
from xml.dom import Node


if __name__ == "__main__":
    #The following exercises the code as demonstrated in the article
    DOC1 = """<?xml version='1.0' encoding='iso-8859-1'?>
<verse>
  <attribution>Louis MacNeice</attribution>
  <line>The Laird o' Phelps spent Hogmanay declaring he was sober,</line>
  <line>Counted his feet to prove the fact and found he had one foot over.</line>
  <line>Mrs. Carmichael had her fifth, looked at the job with repulsion,</line>
  <line>Said to the midwife "Take it away; I'm through with overproduction."</line>
</verse>"""

    DOC2 = """<?xml version='1.0' encoding='iso-8859-1'?>
<nest>
  <bird>cheep</bird> in the
    <nest>
      <bird>cheep</bird> in the <nest>bird</nest>
    </nest>
  <nest>bird</nest> upon a <tree><nest>bird</nest></tree>
</nest>
"""

    DOC3 = """<?xml version="1.0" encoding="iso-8859-1"?>
<labels>
  <label added="2003-06-20">
    <quote>
      <!-- Mixed content -->
      <emph>Midwinter Spring</emph> is its own season&#8230;
    </quote>
    <name>Thomas Eliot</name>
    <address>
      <street>3 Prufrock Lane</street>
      <city>Stamford</city>
      <state>CT</state>
    </address>
  </label>
  <label added="2003-06-10">
    <name>Ezra Pound</name>
    <address>
      <street>45 Usura Place</street>
      <city>Hailey</city>
      <state>ID</state>
    </address>
  </label>
</labels>
    """

    from Ft.Xml.Domlette import NonvalidatingReader
    doc1 = NonvalidatingReader.parseString(DOC1, "http://dummy.ns")
    doc2 = NonvalidatingReader.parseString(DOC2, "http://dummy.ns")
    doc3 = NonvalidatingReader.parseString(DOC3, "http://dummy.ns")

    print "*"*10, "Listing 3", "*"*10
    print "Get elements by tag name:"
    for node in domtools.get_elements_by_tag_name(doc1, 'line'):
        print node
    print "Get elements by tag name ns:"
    for node in domtools.get_elements_by_tag_name_ns(doc1, None, 'line'):
        print node

    from Ft.Xml.XPath import Evaluate, Context
    from Ft.Xml.Domlette import GetAllNs

    def get_elements_by_tag_name_ns_xpath(node, name):
        #I need the context node and a set of namespace mappings
        #In case they use a QName for name e.g. ns:local
        #Use the dictionary of all namespace mappings on the given node
        context = Context.Context(node, GetAllNs(node))
        #Now do an XPath evaluation of an expression that searches
        #The descendant-or-self axis for all nodes whose name
        #matches the given
        return Evaluate(".//" + name, context=context)

    print "Get elements by tag name (XPath):"
    for node in get_elements_by_tag_name_ns_xpath(doc1, 'line'):
        print node

    def get_elements_by_tag_name_ns_raw(node, ns, local):
        if node.nodeType == Node.ELEMENT_NODE and \
           node.namespaceURI == ns and node.localName == local:
            result = [node]
        else:
            result = []
        for child in node.childNodes:
            result.extend(get_elements_by_tag_name_ns_raw(child, ns, local))
        return result

    print "Get elements by tag name (Raw recursion):"
    for node in get_elements_by_tag_name_ns_raw(doc2, None, 'nest'):
        print node

    print "abs_path test"
    print repr(domtools.abs_path(doc3))
    node = doc3.documentElement
    print repr(domtools.abs_path(node))
    node = domtools.get_first_element_by_tag_name_ns(doc3, None, 'label')
    print repr(domtools.abs_path(node))
    node = list(domtools.get_elements_by_tag_name_ns(doc3, None, 'label'))[1]
    print repr(domtools.abs_path(node))
    node = node.attributes[(None, 'added')]
    print repr(domtools.abs_path(node))

    NAME = "GAMES"
    NAME = "v"
    NAME = "LINE"
    BIG_FILE_NAMES = ["1998statistics.xml", "ot.xml", "hamlet.xml"]
    for fname in BIG_FILE_NAMES:
        try:
            big_doc = NonvalidatingReader.parseUri(fname)
        except:
            print "Unable to load", fname
            continue
        element_count = len(
            domtools.get_elements_by_tag_name_ns_raw(big_doc, None, NAME)
            )
        print "Number of", NAME, "elements found:", element_count
        import time
        start = time.time()
        for i in range(10):
            for node in domtools.get_elements_by_tag_name_ns(big_doc, None, NAME):
                pass
        end = time.time()
        print "Generator getElementsByTagNameNS: %.2f"%(end-start)

        start = time.time()
        for i in range(10):
            for node in domtools.get_elements_by_tag_name_ns_xpath(big_doc, NAME):
                pass
        end = time.time()
        print "XPath getElementsByTagNameNS: %.2f"%(end-start)

        start = time.time()
        for i in range(10):
            for node in domtools.get_elements_by_tag_name_ns_raw(big_doc, None, NAME):
                pass
        end = time.time()
        print "Raw recursion getElementsByTagNameNS: %.2f"%(end-start)





#Tests examples given in the manual

import unittest
import amara
from amara import bindery, binderytools
from Ft.Xml import InputSource
from Ft.Lib import Uri

XBEL_PATH = "../../demo/xbel.xml"

class TestManualExamples(unittest.TestCase):
    def setUp(self):
        #Really not used, yet
        self.isrc_factory = InputSource.DefaultFactory
        return

    def _input_source_for_file(self, fname):
        #Create a URI from a filename the right way
        file_uri = Uri.OsPathToUri(fname, attemptAbsolute=1)
        isrc = self.isrc_factory.fromUri(file_uri)
        return isrc

    def testElementPresence(self):
        #isrc = self._input_source_for_file(XBEL_PATH)
        binding = amara.parse(XBEL_PATH)
        self.assertEqual(len(list(binding.xbel)), 1)
        self.assert_(hasattr(binding, 'xbel'))
        self.assertEqual(len(list(binding.xbel.title)), 1)
        self.assert_(hasattr(binding.xbel, 'title'))
        self.assertEqual(len(list(binding.xbel.folder)), 3)
        self.assert_(hasattr(binding.xbel, 'folder'))
        self.assert_(hasattr(binding.xbel.folder, 'title'))
        self.assert_(hasattr(binding.xbel.folder, 'bookmark'))
        self.assert_(hasattr(binding.xbel.folder[1], 'folder'))
        self.assert_(hasattr(binding.xbel.folder.bookmark, 'title'))
        #self.assertEqual(binding.xbel)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testAttributes(self):
        binding = amara.parse(XBEL_PATH)
        self.assert_(hasattr(binding.xbel.folder.bookmark, 'href'))
        self.assertEqual(binding.xbel.folder.bookmark.href, u'http://4suite.org/')
        return

    def testContent(self):
        binding = amara.parse(XBEL_PATH)
        self.assertEqual(unicode(binding.xbel.title), u'Some 4Suite Bookmarks')
        self.assertEqual(unicode(binding.xbel.folder.title), u'Main links')
        self.assertEqual(unicode(binding.xbel.folder.bookmark.title), u'4Suite home page')
        self.assertEqual(len(binding.xbel.folder.bookmark.xml_children), 3)
        self.assertEqual(unicode(binding.xbel.folder.bookmark), u'\n      4Suite home page\n    ')
        self.assertEqual(binding.xbel.folder.bookmark.xml_child_text, u'\n      \n    ')
        self.assertEqual(binding.xbel.folder.bookmark.xml_children[0], u'\n      ')
        return

    def testWsStripElementRule(self):
        strip = [u"/*"]
        custom_rule = binderytools.ws_strip_element_rule(strip)
        binder = bindery.binder()
        binder.preserve_comments = True
        binding = amara.parse(XBEL_PATH, rules=[custom_rule], binderobj=binder)
        #binder.add_rule(custom_rule)
        #binding = binder.read_xml(isrc)
        self.assertEqual(len(binding.xbel.xml_children), 5)
        return


if __name__ == '__main__':
    unittest.main()




import sys
import unittest
from util import run_stron

ENCODING = 'utf-8'


STRON1 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Abstract test</title>
  <pattern abstract="true" id="abs1">
    <rule context="$a">
      <assert test="$b"><name path="$c"/> and <value-of select="$d"/></assert>
    </rule>
  </pattern>

  <pattern name="concrete1" is-a="abs1">
    <param formal="a" actual="actual-a"/>
    <param formal="b"  actual="actual-b"/>
    <param formal="c"  actual="actual-c"/>
    <param formal="d"  actual="actual-d"/>
  </pattern>
</schema>
'''

STRON2 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Abstract test</title>
  <pattern abstract="true" id="abs1">
    <rule context="$a">
      <assert test="$aa"><name path="$aaa"/> and <value-of select="$aaaa"/></assert>
    </rule>
  </pattern>

  <pattern name="concrete1" is-a="abs1">
    <param formal="a" actual="actual-a"/>
    <param formal="aa"  actual="actual-b"/>
    <param formal="aaa"  actual="actual-c"/>
    <param formal="aaaa"  actual="actual-d"/>
  </pattern>
</schema>
'''

INSTANCE1 = u'<actual-a><actual-b/><actual-c/><actual-d>boo</actual-d></actual-a>'.encode(ENCODING)

INSTANCE2 = u'<actual-a><actual-c/><actual-d>boo</actual-d></actual-a>'.encode(ENCODING)


def normalize_text(text):
    #normalize text, a bit slow for large files with much non-ws
    return reduce(lambda a, b: (a and a[-1].isspace() and b.isspace()) and a or a+b, text, '')


class TestBasics(unittest.TestCase):
    def setUp(self):
        #self.isrc_factory = InputSource.DefaultFactory
        return

    def test1_1(self):
        output = run_stron(STRON1, INSTANCE1)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Abstract test\nProcessing pattern: concrete1\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def test1_2(self):
        output = run_stron(STRON1, INSTANCE2)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Abstract test\nProcessing pattern: concrete1\nAssertion failure:\nactual-c and boo\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def test2_1(self):
        output = run_stron(STRON2, INSTANCE1)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Abstract test\nProcessing pattern: concrete1\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def test2_2(self):
        output = run_stron(STRON2, INSTANCE2)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Abstract test\nProcessing pattern: concrete1\nAssertion failure:\nactual-c and boo\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return


if __name__ == '__main__':
    unittest.main()




#Actual demo starts 12 lines below...
import sys
import os
#Make sure we can find needed source files
if not os.access('monty.xml', os.F_OK):
    os.chdir('..')
    if not os.access('monty.xml', os.F_OK):
        print "This file is meant to be run from the demo directory (e.g. same directory as 'monty.xml' or in the demo/bindery directory"
        sys.exit(1)

#Actual demo starts here:
#
# pushdom
#

from amara import domtools
for docfrag in domtools.pushdom('/labels/label', source='labels1.xml'):
    label = docfrag.firstChild
    name = label.xpath('string(name)')
    city = label.xpath('string(address/city)')
    if name.lower().find('eliot') != -1:
        print city.encode('utf-8')

#
# abspath
#

from amara import domtools
from Ft.Xml.Domlette import NonvalidatingReader
from Ft.Lib import Uri
file_uri = Uri.OsPathToUri('labels1.xml', attemptAbsolute=1)
doc = NonvalidatingReader.parseUri(file_uri)

print domtools.abs_path(doc)
print domtools.abs_path(doc.documentElement)
for node in doc.documentElement.childNodes:
    print domtools.abs_path(node)




#Tests XSLT transforms on bindery nodes

import unittest
import amara

from xpath import LABELS, XBEL, RSS

IDENTITY = """\
<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
>
  <xsl:output method="xml" omit-xml-declaration="yes"/>
  <xsl:template match="@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>"""

import sys

XMLDECL = '<?xml version="1.0" encoding="UTF-8"?>\n'

class TestSerialization(unittest.TestCase):
    XML1 = '<doc><foo/></doc>'
    XML2 = '<doc xmlns:a="urn:bogus:a"><hidden attr="b:beta"/></doc>'
    XML3 = '<a:doc xmlns:a="urn:bogus:a"><a:foo/></a:doc>'
    def testAddNsDecl1(self):
        EXPECTED = XMLDECL + '<doc xmlns:a="urn:bogus:a"><foo/></doc>'
        doc = amara.parse(self.XML1)
        result = doc.xml(force_nsdecls={u'a': u'urn:bogus:a'})
        self.assertEqual(EXPECTED, result)
        return

    def testAddNsDecl2(self):
        #Remember that declarations in the doc always trump forced decls, including the
        #Implicit declaration of null default namespace
        EXPECTED = XMLDECL + '<doc><foo/></doc>'
        doc = amara.parse(self.XML1)
        result = doc.xml(force_nsdecls={None: u'urn:bogus:default'})
        self.assertEqual(EXPECTED, result)
        return

    def testAddNsDecl3(self):
        EXPECTED = XMLDECL + '<a:doc xmlns="urn:bogus:default" xmlns:a="urn:bogus:a"><a:foo/></a:doc>'
        doc = amara.parse(self.XML3)
        result = doc.xml(force_nsdecls={None: u'urn:bogus:default'})
        self.assertEqual(EXPECTED, result)
        return

    def testQnamesInContent1(self):
        EXPECTED = XMLDECL + '<doc xmlns:b="urn:bogus:b"><hidden attr="b:beta"/></doc>'
        doc = amara.parse(self.XML2)
        result = doc.xml(force_nsdecls={u'b': u'urn:bogus:b'})
        self.assertEqual(EXPECTED, result)
        return

    def testQnamesInContent2(self):
        EXPECTED = XMLDECL + '<doc xmlns:a="urn:bogus:a" xmlns:b="urn:bogus:b"><hidden attr="b:beta"/></doc>'
        doc = amara.parse(self.XML2)
        result = doc.xml(force_nsdecls={u'a': u'urn:bogus:a', u'b': u'urn:bogus:b'})
        self.assertEqual(EXPECTED, result)
        return


if __name__ == '__main__':
    unittest.main()




import sys
from Ft.Xml import Sax
from Ft.Lib import Uri
from amara import saxtools

XHTML_NS = 'http://www.w3.org/1999/xhtml'

class xhtmlgen_consumer:
    """
    Encapsulation of a set of semi-co-routines designed to handle SAX events
    """
    def __init__(self):
        self.event = None
        self.top_dispatcher = {
           	(saxtools.START_ELEMENT, XHTML_NS, u'html'):
            self.handle_html,
            }
        return

    def handle_html(self, end_condition):
        dispatcher = {
            (saxtools.START_ELEMENT, XHTML_NS, u'head'):
            self.handle_head,
            (saxtools.START_ELEMENT, XHTML_NS, u'body'):
            self.handle_body,
            }
        #Initial call corresponds to the start html element
        curr_gen = None
        yield None
        while (curr_gen is not None) or (not self.event == end_condition):
            curr_gen = saxtools.tenorsax.event_loop_body(dispatcher, curr_gen, self.event)
            yield None
        #Element closed.  Wrap up
        raise StopIteration

    def handle_head(self, end_condition):
        dispatcher = {
            (saxtools.START_ELEMENT, XHTML_NS, 'title'):
            self.handle_title,
            }
        curr_gen = None
        print "Document head started."
        yield None
        while (curr_gen is not None) or (not self.event == end_condition):
            curr_gen = saxtools.tenorsax.event_loop_body(dispatcher, curr_gen, self.event)
            yield None
        #Element closed.  Wrap up
        print "Document head complete."
        raise StopIteration

    def handle_title(self, end_condition):
        yield None
        title = u''
        #No curr-gen because there are no child elements
        while not self.event == end_condition:
            if self.event[0] == saxtools.CHARACTER_DATA:
                title += self.params
            yield None
        #Element closed.  Wrap up
        print "Document title:", title
        raise StopIteration

    def handle_meta(self, end_condition):
        name = self.params.get((None, 'name'))
        content = self.params.get((None, 'content'))
        print "Meta name:", name, " content:"
        print content
        yield None
        return

    def handle_body(self, end_condition):
        dispatcher = {
            (saxtools.START_ELEMENT, XHTML_NS, 'p'):
            self.handle_p,
            }
        curr_gen = None
        print "Document body started."
        yield None
        while (curr_gen is not None) or (not self.event == end_condition):
            curr_gen = saxtools.tenorsax.event_loop_body(dispatcher, curr_gen, self.event)
            yield None
        #Element closed.  Wrap up
        print "Document body complete."
        raise StopIteration

    def handle_p(self, end_condition):
        yield None
        content = u''
        #No curr-gen because child elements are ignored
        while not self.event == end_condition:
            if self.event[0] == saxtools.CHARACTER_DATA:
                content += self.params
            yield None
        #Element closed.  Wrap up
        print "Document content para:", content
        raise StopIteration


if __name__ == "__main__":
    try:
        infile = sys.argv[1]
    except IndexError:
        infile = '../../demo/simple-xhtml.xml'
    parser = Sax.CreateParser()
    consumer = xhtmlgen_consumer()
    handler = saxtools.tenorsax(consumer)
    parser.setContentHandler(handler)
    parser.parse(Uri.OsPathToUri(infile))




"Use Pushbind to scour the Old testament for all verses containing the word 'begat' and print them"

#Download ot.xml (part of http://www.ibiblio.org/bosak/xml/eg/religion.2.00.xml.zip)

import gc
import time
import amara

for v in amara.pushbind('ot.xml', u'v'):
    text = unicode(v)
    if text.find('begat') != -1:
         print text.encode('utf-8')
         gc.collect()

#print "MAIN CODE COMPLETED.  WAITING 10 SECONDS"
#Allow for checking memory reclamation
#time.sleep(10)
#print "FORCING GARBAGE COLLECTION..."
#gc.collect()
#print "WAITING 10 SECONDS..."
#time.sleep(10)




"""
A Web-based inspector of an XML document, using CherryPy and Amara
CherryPy is a brilliant and simple Web framework for Python
For more on CherryPy: http://www.cherrypy.org/
"""

import cStringIO

from amara import binderytools
from Ft.Xml.Domlette import Print, PrettyPrint
from xml.sax.saxutils import escape
from cherrypy import cpg


class xml_inspector:
    def __init__(self):
        self.url = None
        return

    def index(self):
        # Ask for the user's name.
        return '''
            <form action="get_expression" method="GET">
            <div>
            What XML file URL would you like to query
            <input type="text" name="url"/>
            <input type="submit" />
            </div>
            <div>As an example, try entering URL
            "http://uche.ogbuji.net/tech/4Suite/amara/monty.xml"
            and then querying for "monty.python"
            </div>
            </form>
        '''

    index.exposed = True

    def get_expression(self, url=None):
        if not url:
            url = self.url
        if url:
            #Don't re-retrieve the doc if it's already loaded
            if url != self.url:
                self.url = url
                self.doc = binderytools.bind_uri(url)
            return '''
            <form action="show_result" method="GET">
            Enter a Python expression for the XML query
            <input type="text" name="query"/>
            <input type="submit" />
            </form>
            '''
        else:
            # No URL was specified
            return "Please <a href='./'>go back</a> and enter a URL"

    get_expression.exposed = True


    def show_result(self, query=None):
        if query:
            stream = cStringIO.StringIO()
            PrettyPrint(eval("self.doc." + query), stream=stream)
            result = cStringIO.StringIO()
            result.write("<div>Resulting XML fragment:</div><div style='border: thin blue solid'>")
            result.write(escape(stream.getvalue()))
            result.write("</div>")
            result.write("<div>Please <a href='./get_expression'>go back</a> to try another expression, or <a href='./'>all the way back</a> to enter a new XML URL</div>")
            return result.getvalue()
        else:
            # No URL was specified
            return "Please <a href='./get_expression'>go back</a> and enter a URL"

    show_result.exposed = True


cpg.root = xml_inspector()
cpg.server.start(configFile='cherrypy.conf')




"""
Convenience functions for using the Amara bindery.
"""

#
# Utility functions for common binding usage
#

import re
import time
from datetime import time as dt_time
from datetime import datetime, timedelta
from Ft.Xml import InputSource
from Ft.Xml import SplitQName
from Ft.Lib import Uri
from amara import bindery, PI_BINDING
from amara import domtools, saxtools
from Ft.Xml.XPath import Conversions
from xml.dom import Node

#FIXME: Use 4Suite L10N
def _(t): return t

try:
    #From http://labix.org/python-dateutil
    from dateutil.parser import parse as dateparse
    from dateutil.tz import tzlocal
    #Make sure parsed datatimes always have some timezone info
    DEFAULT = datetime(2000, 1, 1, tzinfo=tzlocal())

    def parse_isodate(st):
        try:
            dt = dateparse(st, default=DEFAULT)
            return dt
        except ValueError:
            return None
    parse_isotime = parse_isodate
except ImportError:
    from dateutil_standins import *

#
# Bindery convenience functions
#

def setup_binder_(rules, binderobj, prefixes, validate, binding_classes):
    if binderobj is None:
        binderobj = bindery.binder(prefixes=prefixes)
    for rule in rules:
        binderobj.add_rule(rule)
    if binding_classes:
        for ns, local in binding_classes :
            class_ = binding_classes[ns, local]
            if ns == PI_BINDING:
                binderobj.set_pi_binding_class(local, class_)
            else:
                binderobj.set_binding_class(ns, local, class_)
    return binderobj


def bind_uri(uri, rules=None, binderobj=None, prefixes=None, validate=False,
             binding_classes=None):
    """
    Create a binding from XML retrieved from a URI
    rules - a list of bindery rule objects to fine-tune the binding
    binderobj - optional binder object to control binding details,
                the default is None, in which case a binder object
                will be created
    prefixes - dictionary mapping prefixes to namespace URIs
               the default is None
    """
    rules = rules or []
    #Create an input source for the XML
    isrc_factory = InputSource.DefaultFactory
    isrc = isrc_factory.fromUri(uri)
    binderobj = setup_binder_(rules, binderobj, prefixes, validate, binding_classes)

    #Now bind from the XML given in the input source
    binding = binderobj.read_xml(isrc, validate)
    return binding


def bind_file(fname, rules=None, binderobj=None, prefixes=None, validate=False,
              binding_classes=None):
    """
    Create a binding from XML read from a file
    rules - a list of bindery rule objects to fine-tune the binding
    binderobj - optional binder object to control binding details,
                the default is None, in which case a binder object
                will be created
    prefixes - dictionary mapping prefixes to namespace URIs
               the default is None
    """
    #Create an input source for the XML
    isrc_factory = InputSource.DefaultFactory
    #Create a URI from a filename the right way
    file_uri = Uri.OsPathToUri(fname, attemptAbsolute=1)
    binding = bind_uri(file_uri, rules=rules, binderobj=binderobj,
                       prefixes=prefixes, validate=validate,
                       binding_classes=binding_classes)
    return binding

DEFAULT_URI = 'urn:bogus:unknown-resource-uri'

def bind_stream(stream, uri=None, rules=None, binderobj=None, prefixes=None,
                validate=False, binding_classes=None):
    """
    Create a binding from XML retrieved from a file-like object
    rules - a list of bindery rule objects to fine-tune the binding
    binderobj - optional binder object to control binding details,
                the default is None, in which case a binder object
                will be created
    prefixes - dictionary mapping prefixes to namespace URIs
               the default is None
    """
    rules = rules or []
    #Create an input source for the XML
    isrc_factory = InputSource.DefaultFactory
    isrc = isrc_factory.fromStream(stream, uri or DEFAULT_URI)

    binderobj = setup_binder_(rules, binderobj, prefixes, validate, binding_classes)

    #Now bind from the XML given in the input source
    binding = binderobj.read_xml(isrc, validate)
    return binding

def bind_string(st, uri=None, rules=None, binderobj=None, prefixes=None, validate=False, binding_classes=None):
    """
    Create a binding from XML in string (NOT unicode) form
    rules - a list of bindery rule objects to fine-tune the binding
    binderobj - optional binder object to control binding details,
                the default is None, in which case a binder object
                will be created
    prefixes - dictionary mapping prefixes to namespace URIs
               the default is None
    """
    import cStringIO
    stream = cStringIO.StringIO(st)
    binding = bind_stream(stream, uri=uri, rules=rules, binderobj=binderobj,
                       prefixes=prefixes, validate=validate, binding_classes=binding_classes)
    return binding


def create_document(qname=None, ns=None, content=None, attributes=None,
                    pubid=None, sysid=None, prefixes=None):
    """
    Create a document, with optional convenience arguments to create
    top-level information items
    qname - optional QName of the document element (which will be created)
            if QName is not given, no other arguments may be given
    ns - optional namespace of the document element
    content - optional Unicode that makes up text content to be set
              as the child of the document element
    pubid - optional public ID of the doctype (to be set on the document)
    sysid - optional system ID of the doctype (to be set on the document)
    prefixes - optional set of additional, preferred prefix/namespace mappings
               for the document to track
    """
    if not qname:
        doc = bindery.root_base()
        return doc
    doc = bindery.root_base(doctype_name=qname, pubid=pubid, sysid=sysid)
    prefix, local = SplitQName(qname)
    prefixes = prefixes or {}
    prefixes.update(ns and {prefix: ns} or {})
    doc.xmlns_prefixes = prefixes
    doc_elem = doc.xml_create_element(qname, ns, attributes=attributes)
    doc.xml_append(doc_elem)
    if content:
        doc_elem.xml_append(content)
    return doc


#
#Optional bindery rules
#

class xpattern_rule_base(object):
    def __init__(self, xpatterns):
        if isinstance(xpatterns, str) or isinstance(xpatterns, unicode) :
            xpatterns = [xpatterns]
        self.xpatterns = xpatterns
        return

    def add_rule_hook(self, binder):
        binder.xpatterns.extend(self.xpatterns)
        return

    def match(self, binder):
        #print binder.event
        #print (binder.state_machine.entering_xpatterns, self.xpatterns)
        for xp in binder.state_machine.entering_xpatterns:
            if xp in self.xpatterns: return True
        return False
    
    def dom_match(self, node, binder):
        #Currently not used
        plist = PatternList(self.xpatterns, binder.prefixes)
        #not not trick acts like a cast to boolean
        context = Context(node.ownerDocument,
                          processorNss=binder.prefixes)
        return not not plist.lookup(node, context)


class simple_string_element_rule(xpattern_rule_base):
    """
    An Amara bindery rule.  Bindery rules allow developers to customize how
    XML documents are translated to Python objects.
    
    This rule is pattern based.  Elements that match the pattern will
    be bound as simple Python unicode objects, rather than full
    element objects, saving memory.
    
    There is no default pattern.  You must specify one.
    """
    event_type = saxtools.START_ELEMENT
    priority = -30

    def apply(self, binder):
        if binder.event_completely_handled:
            #Then another rule has already created an instance
            return
        if not self.match(binder):
            return
        #Manage a stack of elements that match this start, so that we know
        #When we've really met our matching end element
        #Set it to 1 because we want it to roll to 0 when we meed the matching end element
        self.stack_depth = 1
        
        parent = binder.binding_stack[-1]
        if not parent: return

        (dummy, qname, ns, local, attributes) = binder.event
        prefix = qname[:qname.rindex(local)][:-1]
        ename = binder.naming_rule.xml_to_python(qname, ns)

        #Inset a trigger for handling child characters
        def handle_char(binder):
            if binder.event_completely_handled:
                return
            (dummy, text) = binder.event
            #Append current char data to any already added
            cdata = getattr(parent, ename, u'') + text
            setattr(parent, ename, cdata)
            binder.event_completely_handled = True
            return
        handle_char.priority = -31
        binder.add_rule(handle_char, saxtools.CHARACTER_DATA)

        #Inset a trigger for handling child start elements
        def handle_start(binder):
            #Skip child elements
            binder.event_completely_handled = True
            return
        handle_start.priority = -31
        binder.add_rule(handle_start, saxtools.START_ELEMENT)

        #Inset a trigger for handling the matching end element
        def handle_end(binder):
            (dummy, qname, ns, local) = binder.event
            self.stack_depth -= 1
            if not self.stack_depth:
                binder.remove_rule(handle_end, saxtools.END_ELEMENT)
                binder.remove_rule(handle_char, saxtools.CHARACTER_DATA)
                binder.remove_rule(handle_start, saxtools.START_ELEMENT)
            binder.event_completely_handled = True
            return
        handle_end.priority = -31
        binder.add_rule(handle_end, saxtools.END_ELEMENT)
        binder.event_completely_handled = True
        return


class omit_element_rule(xpattern_rule_base):
    """
    An Amara bindery rule.  Bindery rules allow developers to customize how
    XML documents are translated to Python objects.
    
    This rule is pattern based.  Elements that match the pattern will
    be ignored and not bound to any object at all, saving memory.
    
    There is no default pattern.  You must specify one.
    """
    event_type = saxtools.START_ELEMENT
    priority = -40

    def apply(self, binder):
        if not self.match(binder):
            return
        if binder.event_completely_handled:
            #Then another rule has already created an instance
            return

        #Manage a stack of elements that match this start, so that we know
        #When we've really met our matching end element
        #Set it to 1 because we want it to roll to 0 when we meed the matching end element
        self.stack_depth = 1
        
        #Inset a trigger for handling child characters
        def handle_char(binder):
            #Skip child characters
            binder.event_completely_handled = True
            return
        handle_char.priority = -41
        binder.add_rule(handle_char, saxtools.CHARACTER_DATA)

        #Inset a trigger for handling child start elements
        def handle_start(binder):
            self.stack_depth += 1
            binder.event_completely_handled = True
            return
        handle_start.priority = -41
        binder.add_rule(handle_start, saxtools.START_ELEMENT)

        #Inset a trigger for handling the matching end element
        def handle_end(binder):
            self.stack_depth -= 1
            if not self.stack_depth:
                binder.remove_rule(handle_end, saxtools.END_ELEMENT)
                binder.remove_rule(handle_char, saxtools.CHARACTER_DATA)
                binder.remove_rule(handle_start, saxtools.START_ELEMENT)
            binder.event_completely_handled = True
            return
        handle_end.priority = -41
        binder.add_rule(handle_end, saxtools.END_ELEMENT)
        binder.event_completely_handled = True
        return

NODETYPE_LOOKUP = {Node.COMMENT_NODE: saxtools.COMMENT,
                   Node.PROCESSING_INSTRUCTION_NODE: saxtools.PI,
                   Node.TEXT_NODE: saxtools.CHARACTER_DATA}

class omit_nodetype_rule(object):
    """
    An Amara bindery rule.  Bindery rules allow developers to customize how
    XML documents are translated to Python objects.
    
    This rule allows you to omit all comments, or all processing instructions,
    or all text nodes from the binding, for convenience or memory savings.
    """
    priority = -40
    def __init__(self, nodetype):
        self.nodetypes = nodetype
        self.event_type = NODETYPE_LOOKUP[nodetype]
        return

    def match(self, binder):
        return True
    
    def apply(self, binder):
        if not self.match(binder):
            return
        if binder.event_completely_handled:
            #Then another rule has already created an instance
            return

        binder.event_completely_handled = True
        return


class element_skeleton_rule(xpattern_rule_base):
    """
    An Amara bindery rule.  Bindery rules allow developers to customize how
    XML documents are translated to Python objects.
    
    This rule is pattern based.  Elements that match the pattern and their
    children  will be bound as elements only, with no child character data
    preserved.  Use this when you only care about the element structure and
    want to save resources by not loading the character data.
    
    The default pattern is '*', meaning all elements.
    """
    event_type = saxtools.START_ELEMENT
    priority = 20

    def __init__(self, xpatterns=None):
        if not xpatterns:
            xpatterns = [u'*']
        xpattern_rule_base.__init__(self, xpatterns)
        #Track how deep we are within a matched element event
        #0 means not within a matched element event
        self.stack_depth = 0
        return
    
    def apply(self, binder):
        if binder.event_completely_handled:
            #Then another rule has already created an instance
            return
        if not self.match(binder):
            return
        #Manage a stack of elements that match this start, so that we know
        #When we've really met our matching end element
        #Set it to 1 because we want it to roll to 0 when we meed the matching end element
        self.stack_depth = 1
        
        (dummy, qname, ns, local, attributes) = binder.event
        prefix = qname[:qname.rindex(local)][:-1]
        ename = binder.naming_rule.xml_to_python(qname, ns)

        #Inset a trigger for handling child characters
        def handle_char(binder):
            binder.event_completely_handled = True
            return
        handle_char.priority = -50
        binder.add_rule(handle_char, saxtools.CHARACTER_DATA)

        #Inset a trigger for handling child start elements
        def handle_start(binder):
            binder.event_completely_handled = False
            self.stack_depth += 1
            return
        handle_start.priority = 20
        binder.add_rule(handle_start, saxtools.START_ELEMENT)

        #Inset a trigger for handling the matching end eleemnt
        def handle_end(binder):
            self.stack_depth -= 1
            if self.stack_depth == 0:
                #Back to original matching end element
                binder.remove_rule(handle_char, saxtools.CHARACTER_DATA)
                binder.remove_rule(handle_end, saxtools.END_ELEMENT)
                binder.remove_rule(handle_start, saxtools.START_ELEMENT)
                binder.event_completely_handled = False
                return
            return
        handle_end.priority = 20
        binder.add_rule(handle_end, saxtools.END_ELEMENT)
        binder.event_completely_handled = True
        return


class ws_strip_element_rule(xpattern_rule_base, bindery.default_element_rule):
    """
    An Amara bindery rule.  Bindery rules allow developers to customize how
    XML documents are translated to Python objects.
    
    This rule is pattern based.  Elements that match the pattern and
    descendants will have text children stripped if they are pure whitespace.
    In other words
    <a> <b/> <c/> </a>
    will end up with the same binding as
    <a> <b/> <c/> </a>
    but not
    <a> <b/> x <c/> </a> <!-- ' x ' is not pure whitespace -->
    Stripping whitespace saves memory and can simplify resulting bindings,
    but be sure the whitespace doesn't have a significant meeaning in the
    document.
    
    The default pattern is '*', meaning all elements.
    """
    event_type = saxtools.START_ELEMENT
    priority = -20
    
    def __init__(self, xpatterns=None):
        if not xpatterns:
            xpatterns = [u'*']
        xpattern_rule_base.__init__(self, xpatterns)
        return

    def apply(self, binder):
        if not self.match(binder):
            return
        if binder.event_completely_handled:
            #Then another rule has already created an instance
            return
        self.stack_depth = 1
        
        #Inset a trigger for handling child characters
        def handle_char(binder):
            if binder.event_completely_handled:
                return
            parent = binder.binding_stack[-1]
            if not parent: return
            (dummy, text) = binder.event
            if text.strip():
                parent.xml_children.append(text)
            binder.event_completely_handled = True
            return
        handle_char.priority = -21
        binder.add_rule(handle_char, saxtools.CHARACTER_DATA)

        #Inset a trigger for handling child start elements
        def handle_start(binder):
            self.stack_depth += 1
            if binder.event_completely_handled:
                return
            parent = binder.binding_stack[-1]
            if not parent: return
            (dummy, qname, ns, local, attributes) = binder.event
            prefix = qname[:qname.rindex(local)][:-1]
            ename = binder.naming_rule.xml_to_python(local, ns, prefix)

            instance = bindery.create_element(binder.naming_rule, qname, ns, ename)
            bindery.bind_attributes(instance, parent, ename, binder)
            instance = bindery.bind_instance(instance, parent)
            binder.binding_stack.append(instance)
            binder.event_completely_handled = True
            return
        handle_start.priority = -21
        binder.add_rule(handle_start, saxtools.START_ELEMENT)

        #Inset a trigger for handling the matching end eleemnt
        def handle_end(binder):
            self.stack_depth -= 1
            if not self.stack_depth:
                binder.remove_rule(handle_end, saxtools.END_ELEMENT)
                binder.remove_rule(handle_char, saxtools.CHARACTER_DATA)
                binder.remove_rule(handle_start, saxtools.START_ELEMENT)
            if not binder.event_completely_handled:
                #if binder.binding_stack[-1] is instance:
                binder.binding_stack.pop()
            binder.event_completely_handled = True
            return
        handle_end.priority = -21
        binder.add_rule(handle_end, saxtools.END_ELEMENT)

        handle_start(binder)

        return


def infer_data_from_string(value):
    if not isinstance(value, basestring):
        return value
    data_value = None
    try:
        data_value = int(value)
    except (TypeError, ValueError):
        try:
            data_value = float(value)
        except (TypeError, ValueError):
            data_value = parse_isodate(value)
            if not data_value:
                data_value = parse_isotime(value)
    return data_value


class type_inference(xpattern_rule_base):
    """
    An Amara bindery rule.  Bindery rules allow developers to customize how
    XML documents are translated to Python objects.
    
    This rule is pattern based.  Children of elements that match
    the pattern are checked to see if they could be a more specialized
    type, such as int, float or date
    
    The default pattern is '*', meaning all elements.
    """
    event_type = saxtools.START_ELEMENT
    priority = 30

    def __init__(self, xpatterns=None):
        if not xpatterns:
            xpatterns = [u'*']
        xpattern_rule_base.__init__(self, xpatterns)
        return
    
    def apply(self, binder):
        if not self.match(binder):
            return
        self.stack_depth = 1

        #Inset a trigger for handling contained matching elements
        def handle_start(binder):
            self.stack_depth += 1
            (dummy, qname, ns, local, attributes) = binder.event
            #Assume a previous rule handler has added
            #The current instance to the stack
            instance = binder.binding_stack[-1]
            #key_list = [ binder.naming_rule.xml_to_python() for qn in attributes.getQNames() ]
            if hasattr(instance, "xml_attributes"):
                for attr, (qname, ns) in instance.xml_attributes.items():
                    value = getattr(instance, attr)
                    data_value = infer_data_from_string(value)
                    if data_value is not None:
                        setattr(instance, attr, data_value)
            #print "start", self.stack_depth, binder.event[2:4], instance.__class__, binder.binding_stack, instance.xml_attributes
            binder.event_completely_handled = True
            return
        #Rule must be added to the end, after the default (which does the usual
        #element handling)
        handle_start.priority = 30
        binder.add_rule(handle_start, saxtools.START_ELEMENT)
        #binder.rules[saxtools.START_ELEMENT].append(handle_start)
        #print binder.rules
        #Even though we've added it to the rules list, it won't be immediately executed
        handle_start(binder)

        #Inset a trigger for handling the matching end eleemnt
        def handle_end(binder):
            self.stack_depth -= 1
            if not self.stack_depth:
                binder.remove_rule(handle_start, saxtools.START_ELEMENT)
                binder.remove_rule(handle_end, saxtools.END_ELEMENT)
            (dummy, qname, ns, local) = binder.event
            prefix = qname[:qname.rindex(local)][:-1]
            ename = binder.naming_rule.xml_to_python(qname, ns)

            #print "end", self.stack_depth, binder.event[2:4], binder.rules[saxtools.END_ELEMENT]
            instance = binder.binding_stack[-1]
            #Only infer a type for an element if it has all text node children
            #And no attributes
            if ( not hasattr(instance, "xml_attributes") and
                 not [ child for child in instance.xml_children
                             if not isinstance(child, unicode) ] ):
                #No child elements
                value = u"".join(instance.xml_children)
                data_value = infer_data_from_string(value)
                if data_value is None:
                    data_value = value
                parent = instance.xml_parent
                if len(list(getattr(parent, ename))) == 1:
                    parent.__dict__[ename] = data_value
                    i = parent.xml_children.index(instance)
                    parent.xml_children[i] = data_value
                binder.binding_stack.pop()
                binder.event_completely_handled = True
            return
        handle_end.priority = -60
        binder.add_rule(handle_end, saxtools.END_ELEMENT)
        binder.event_completely_handled = True
        return


#General utility functions for bindery

def fixup_namespaces(node):
    """
    reduces namespace clutter in documents by looking for duplicate namespace
    declarations and preferring those set as document prefixes
    """
    doc = node.rootNode
    nss = dict(zip(doc.xmlns_prefixes.values(), doc.xmlns_prefixes.keys()))
    #FIXME: handle attributes as well
    if node.namespaceURI in nss:
        node.xmlnsPrefix = nss[node.namespaceURI]
        node.nodeName = node.prefix and node.prefix + ':' + node.localName or node.localName
    for child in node.xml_child_elements.value(): fixup_namespaces(child)
    return


def quick_xml_scan(source, field, filter=None, display=None, **kwargs):
    """
    Scan through an XML file to extract the first occurrence of single field,
    without loading more of the file than needed to find the data
    field - a Uncode object representing an XSLT pattern
    filter - an optional XPath expression for filtering results from the pattern
    display - an optional XPath expression indicating what value to display, with the matching node from the pattern as the context
    """
    #A n elaboration of
    # http://copia.ogbuji.net/blog/2006-01-08/Recipe__fa
    nodes = pushbind(source, field, **kwargs)
    for node in nodes:
        if not xpath or node.xml_xpath(filter):
            if display:
                #Print specified subset
                result = Conversions.StringValue(node.xml_xpath(display))
            else:
                result = Conversions.StringValue(node)
    return result


#
# Pushbind
#

import os
import xml.sax
from Ft.Xml import Sax
from amara.saxtools import sax2dom_chunker
from Ft.Xml import InputSource
from Ft.Lib import Uri, Uuid
from Ft.Xml.Lib.XmlString import IsXml
from Ft.Xml.XPath import Context, Compile


def pushbind(source, xpatterns, prefixes=None, rules=None, event_hook=None, validate=False):
    """
    Generator that yields subtrees of Python object bindings from XML
    according to the given pattern.  The patterns are must be XSLT patterns
    matching elements.
    
    Example: for an xml document
    
    XML = '<a> <b><c/></b> <b><c/></b> <b><c/></b> </a>'
    
    pushbind('b', string=XML)
    Will return a generator that yields the first, second and third b
    element in turn as Python objects containing the full subtree.
    
    Warning: this function uses threads and only works in Python distros
    with threading support.

    lookahead - how far ahead the SAX engine looks for events before waiting
    for the user to request another subtree from the generator.  0 means
    the SAX engine will just load and queue up object for
    the entire file in the background.
    """
    parser = Sax.CreateParser()
    parser.setFeature(Sax.FEATURE_GENERATOR, True)

    if validate:
        parser.setFeature(xml.sax.handler.feature_validation, True)
    else:
        parser.setFeature(xml.sax.handler.feature_external_pes, False)

    def handle_chunk(node):
        parser.setProperty(Sax.PROPERTY_YIELD_RESULT, node)

    #Create an instance of the chunker
    handler = saxbind_chunker(xpatterns=xpatterns, prefixes=prefixes,
        chunk_consumer=handle_chunk, rules=rules, event_hook=event_hook
    )

    parser.setContentHandler(handler)
    if isinstance(source, InputSource.InputSource):
        pass
    elif hasattr(source, 'read'):
        #Create dummy Uri to use as base
        dummy_uri = 'urn:uuid:'+Uuid.UuidAsString(Uuid.GenerateUuid())
        source = InputSource.DefaultFactory.fromStream(source, dummy_uri)
    elif IsXml(source):
        #Create dummy Uri to use as base
        dummy_uri = 'urn:uuid:'+Uuid.UuidAsString(Uuid.GenerateUuid())
        source = InputSource.DefaultFactory.fromString(source, dummy_uri)
    elif Uri.IsAbsolute(source): #or not os.path.isfile(source):
        source = InputSource.DefaultFactory.fromUri(source)
    else:
        source = InputSource.DefaultFactory.fromUri(Uri.OsPathToUri(source))
    return parser.parse(source)


from amara.binderyxpath import xpath_attr_wrapper
class dummy_element(object):
    nodeType = Node.ELEMENT_NODE
    def __init__(self, name, attribs, root):
        self._root = root
        self.xpathAttributes = [
            xpath_attr_wrapper(
                attribs.getQNameByName((ns, local)), ns, value, self
                )
            for (ns, local), value in attribs.items()
            ]
        #self.attributes = dict([
        #    (1, attr) for attr in self.xpathAttributes
        #    ])
    
    def _rootNode(self):
        return self._root


class saxbind_chunker(bindery.binder, saxtools.sax2dom_chunker):
    """
    Note: Ignores nodes prior to the document element, such as PIs and
    text nodes.  Collapses CDATA sections into plain text
    Only designed to work if you set the feature
      sax.handler.feature_namespaces
    to 1 on the parser you use.

    xpatterns - list of XSLT Patterns.  Only portions of the
        tree within these patterns will be instantiated as DOM (as
        chunks fed to chunk_consumer in sequence)
        If None (the default, a DOM node will be created representing
        the entire tree.

    prefixes - a dictionary of prefix -> namespace name mappings used to
        interpret XPatterns
    
    chunk_consumer - a callable object taking an XML root object.  It will be
        invoked as each chunk is prepared.
    """
    def __init__(self,
                 xpatterns=None,
                 prefixes=None,
                 chunk_consumer=None,
                 rules=None,
                 event_hook=None,
                 ):
        bindery.binder.__init__(self, prefixes)
        #Add patterns from the push match specs
        if isinstance(xpatterns, str) or isinstance(xpatterns, unicode):
            xpatterns = [xpatterns]
        self.xpatterns.extend(xpatterns)
        rules = rules or []
        for rule in rules:
            self.add_rule(rule)
        prefixes = prefixes or {}
        #print self.xpatterns
        self.state_machine = saxtools.xpattern_state_manager(self.xpatterns, prefixes)
        self._chunk_consumer = chunk_consumer
        self.event_hook = event_hook
        return

    #Overridden ContentHandler methods
    def startDocument(self):
        bindery.binder.startDocument(self)
        if self.event_hook:
            try:
                self.event_hook.startDocument()
            except AttributeError:
                pass
        return

    def endDocument(self):
        bindery.binder.endDocument(self)
        if self.event_hook:
            try:
                self.event_hook.endDocument()
            except AttributeError:
                pass
        return

    def startElementNS(self, (ns, local), qname, attribs):
        #print "startElementNS", (self, (ns, local), qname, attribs)
        self.state_machine.event(1, ns, local)
        if not self.state_machine.status():
            if self.event_hook:
                try:
                    self.event_hook.startElementNS((ns, local), qname, attribs)
                except AttributeError:
                    pass
            for attrtest in self.state_machine.attribute_tests:
                root = self.binding_stack[0]
                e = dummy_element(qname, attribs, root)
                matched = attrtest.evaluate(Context.Context(e))
                if matched:
                    #Feed the consumer
                    attr = matched[0]
                    root.xmlns_prefixes = self.prefixes
                    root.xml_append(dummy_element)
                    self._chunk_consumer(attr)
                    #Start over with new doc frag so old memory can be reclaimed
                    self.binding_stack = []
                    self.event = (saxtools.START_DOCUMENT,)
                    self.apply_rules()
            return
        self.event = (saxtools.START_ELEMENT, qname, ns, local, attribs)
        self.apply_rules()
        return

    def endElementNS(self, (ns, local), qname):
        self.state_machine.event(0, ns, local)
        if not self.state_machine.status():
            #print (self.state_machine._state, self.state_machine.entering_xpatterns, self.state_machine.leaving_xpatterns)
            if (self._chunk_consumer and
                self.state_machine.leaving_xpatterns):
                self.event = (saxtools.END_ELEMENT, qname, ns, local)
                self.apply_rules()
                self.event = (saxtools.END_DOCUMENT,)
                self.apply_rules()
                root = self.binding_stack[0]
                elem = root.xml_children[-1]
                del root
                elem.xmlns_prefixes = self.prefixes
                elem.xml_parent = None
                self._chunk_consumer(elem)
                #Start all over with a new container so the old
                #One's memory can be reclaimed
                self.binding_stack = []
                self.event = (saxtools.START_DOCUMENT,)
                self.apply_rules()
            else:
                if self.event_hook:
                    try:
                        self.event_hook.endElementNS((ns, local), qname)
                    except AttributeError:
                        pass
            return
        self.event = (saxtools.END_ELEMENT, qname, ns, local)
        self.apply_rules()
        return

    def characters(self, chars):
        if self.state_machine.status():
            bindery.binder.characters(self, chars)
        else:
            if self.event_hook:
                try:
                    self.event_hook.characters(chars)
                except AttributeError:
                    pass
        return





#IGNORE FOR NOW

"""
Create HTML docs and index from a set of RELAX NG files
"""

import sys
import os
import re
from amara import binderytools

rng_dir = sys.argv[1]
output_file = open(sys.argv[2], 'w')
RNG_FILE_PAT = re.compile('.*\.rng')

PREFIXES = {
    'rng': 'http://relaxng.org/ns/structure/1.0',
    'ega': 'http://examplotron.org/annotations/'
}

from Ft.Xml.Xslt.XmlWriter import XmlWriter
#from Ft.Xml.Xslt.HtmlWriter import HtmlWriter
from Ft.Xml.Xslt.OutputParameters import OutputParameters


class define(object):
    def __init__(self, name):
        self.name = name
        self.elements = []
        self.attributes = []
        self.text_patterns = []
        self.xhtml_fragment = ''

class namespace(object):
    def __init__(self, name):
        self.uri = uri
        self.prefix = prefix
        self.elements = []
        self.attributes = []
        self.xhtml_fragment = ''

class attribute(object):
    def __init__(self, name):
        self.name = name
        self.elements = []
        self.attributes = []
        self.text_patterns = []
        self.xhtml_fragment = ''

class element(object):
    def __init__(self, name):
        self.name = name
        self.elements = []
        self.attributes = []
        self.text_patterns = []
        self.xhtml_fragment = ''

class rngdoc:
def process_rng(path, writer):
    container = binderytools.bind_file(path, prefixes=PREFIXES)
    for define in container.xml_xpath('//rng:define'):
        print define.name
        pass
    #container.xml_xpath(u'//rng:externalRef')
    return


oparams = OutputParameters()
oparams.indent = 'yes'
writer = XmlWriter(oparams, output_file)
writer.startDocument()

for cdir, subdirs, files in os.walk(rng_dir):
    #print "Processing:", cdir, subdirs, files
    for fname in files:
        #if fname.endsWith('.rng')
        if RNG_FILE_PAT.match(fname):
            path = os.path.join(cdir, fname)
            print 'Processing', path
            process_rng(path, writer)

writer.endDocument()




"""
Demonstrate sibling grouping, a common PITA problem.
Specifically inspired by:
http://mail.python.org/pipermail/xml-sig/2005-March/011028.html
"""
import amara
from amara import bindery

XML="""\
<questions>
  <question>
    <content>Why?</content>
    <answer id="1"/>
  </question>
  <question>
    <content>Why?</content>
    <answer id="2"/>
  </question>
</questions>
"""

ANSWER1 = """\
<content>just because</content>
"""

#Subclass from the default binding class
#We're adding a specialized method for accessing content
class answer_proxy(bindery.element_base):
    def __init__(self):
        #Flag to avoid problematic recursion
        #Use leading "xml" convention to avoid any clashes with XML constructs
        self.xml_mutex = False 
        bindery.element_base.__init__(self)
    
    def __getattr__(self, name):
        #Warning: __getattr__ is one of the trickest aspects of Python to
        #get right, and to debug.
        #BTW, you rarely want to use __getattribute__ in this case
        #Overall, always good to be familiar with
        #http://www.python.org/download/releases/2.2.3/descrintro/
        if name == 'content' and not self.xml_mutex:
            #Use Amara API as you wish, in this case grabbing the id attribute
            answer_ob_name = 'ANSWER' + self.id
            try:
                self.xml_mutex = True #Set flag to avoid unwanted recursion
                self.xml_append_fragment(globals()[answer_ob_name])
                self.xml_mutex = False
            except KeyError:
                msg = u'Answer document fragment string %s not found'%answer_ob_name
                elem = self.xml_create_element(u'content', content=msg)
                self.xml_append(elem)
            return self.__dict__[name]
        raise AttributeError('%r has no attribute "%s"'%(self, name))


doc = amara.parse(XML, binding_classes={(None, 'answer'): answer_proxy})

print doc.xml()
print "Accessing answer 1: ", doc.questions.question.answer.content
print "Accessing answer 2: ", doc.questions.question[1].answer.content
print doc.xml()




"""
Use the command line tidy to grab well-formed XHTML as a bindery
"""

import os
import urllib
import sys
from amara.binderytools import *

NS = {u'html': u'http://www.w3.org/1999/xhtml'}
CMDLINES = [
    'java -jar tagsoup.jar',
    'tidy -nq -asxhtml'
    ]

def tidy_bind_url(url, rules=None, binderobj=None, prefixes=None):
    #Add in a prefixed version of the namespace as well, for easier XPath
    prefixes = prefixes or {}
    prefixes.update(NS)
    for cmd in CMDLINES:
        try:
            #print >> sys.stderr, "Trying", cmd
            stream = urllib.urlopen(url)
            pin, pout = os.popen2(cmd)
            pin.write(stream.read())
            pin.close()
            break
        except IOError:
            pass
    doc = pout.read()
    binding = bind_string(doc, url, rules=rules, binderobj=binderobj, prefixes=prefixes)
    return binding


if __name__ == '__main__':
    #Sample usage
    #Internal link checker
    #url = "http://www.w3.org/TR/REC-xml" #XML spec
    #Uche's "Detailed cross-reference of the most important XML standards"
    url = "http://www.ibm.com/developerworks/xml/library/x-stand4/"
    if len(sys.argv) > 1:
        url = sys.argv[1]
    doc = tidy_bind_url(url)
    all_links = doc.xml_xpath(u'//html:a[@href]')
    all_anchors = doc.xml_xpath(u'//html:a[@name]')
    anchor_names = [ a.name for a in all_anchors ]
    internal_refs = [ a.href for a in all_links if a.href.startswith(u'#') ]
    for ref in internal_refs:
        if ref[1:] in anchor_names:
            print ref, "valid"
        else:
            print ref, "invalid"

if False:
    #Another example, change "False" to "True" above to enable
    url = "http://webjay.org/by/chromegat/theclassicnaijajukebox2823229"
    doc = tidy_bind_url(url)
    #Display all links to mp3s (by file extension check)
    links = doc.xml_xpath(u'//html:a[@href]')
    for link in links:
        if link.href.endswith(u'.mp3'):
            print link.href




#Test examples in http://uche.ogbuji.net/tech/pubs/schematron.html
import sys
import unittest
from util import run_stron


ENCODING = 'utf-8'


#Listing 5 with root context hack in place
#Includes bug fix to article listing: removed erroneous addr: prefix in first assert
LISTING5_1 = '''\
<schema xmlns='http://purl.oclc.org/dsdl/schematron'>
        <pattern name="Structural Validation">
                <!-- Use a hack to set the root context.  Necessary because of
                     a bug in the schematron 1.3 meta-transforms. -->
                <rule context="/*">
                        <assert test="../ADDRBOOK">Validation error: there must be an ADDRBOOK element at the root of the document.</assert>
                </rule>
                <rule context="ENTRY">
                        <assert test="count(NAME) = 1">Validation error: <name/> element missing a NAME child.</assert>
                        <assert test="count(ADDRESS) = 1">Validation error: <name/> element missing an ADDRESS child.</assert>
                        <assert test="count(EMAIL) = 1">Validation error: <name/> element missing an EMAIL child.</assert>
                        <assert test="NAME[following-sibling::ADDRESS] and ADDRESS[following-sibling::PHONENUM] and PHONENUM[following-sibling::EMAIL]">Validation error: <name/> must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence</assert>
                        <assert test="count(NAME|ADDRESS|PHONENUM|EMAIL) = count(*)">Validation error: there is an extraneous element child of ENTRY</assert>
                </rule>
                <rule context="PHONENUM">
                        <assert test="@DESC">Validation error: <name/> must have a DESC attribute</assert>
                </rule>
        </pattern>
</schema>'''

#Listing 5 without root context hack
#Includes bug fix to article listing: removed erroneous addr: prefix in first assert
LISTING5_2 = '''\
<schema xmlns='http://purl.oclc.org/dsdl/schematron'>
        <pattern name="Structural Validation">
                <rule context="/">
                        <assert test="ADDRBOOK">Validation error: there must be an ADDRBOOK element at the root of the document.</assert>
                </rule>
                <rule context="ENTRY">
                        <assert test="count(NAME) = 1">Validation error: <name/> element missing a NAME child.</assert>
                        <assert test="count(ADDRESS) = 1">Validation error: <name/> element missing an ADDRESS child.</assert>
                        <assert test="count(EMAIL) = 1">Validation error: <name/> element missing an EMAIL child.</assert>
                        <assert test="NAME[following-sibling::ADDRESS] and ADDRESS[following-sibling::PHONENUM] and PHONENUM[following-sibling::EMAIL]">Validation error: <name/> must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence</assert>
                        <assert test="count(NAME|ADDRESS|PHONENUM|EMAIL) = count(*)">Validation error: there is an extraneous element child of ENTRY</assert>
                </rule>
                <rule context="PHONENUM">
                        <assert test="@DESC">Validation error: <name/> must have a DESC attribute</assert>
                </rule>
        </pattern>
</schema>'''

#Listing 6 with root context hack in place
LISTING6_1 = '''\
<schema xmlns='http://purl.oclc.org/dsdl/schematron'>

        <ns prefix='addr' uri='http://addressbookns.com'/>

        <pattern name="Structural Validation">
                <!-- Use a hack to set the root context.  Necessary because of
                     a bug in the schematron 1.3 meta-transforms. -->
                <rule context="/*">
                        <assert test="../addr:ADDRBOOK">Validation error: there must be an ADDRBOOK element at the root of the document.</assert>
                </rule>
                <rule context="addr:ENTRY">
                        <assert test="count(addr:*) + count(*[contains(namespace-uri(.), 'extension')]) = count(*)">
Validation error: all children of <name/> must either be in the namespace 'http://addressbookns.com' or in a namespace containing the substring 'extension'.
                        </assert>
                        <assert test="count(addr:NAME) = 1">
Validation error: <name/> element missing a NAME child.
                        </assert>
                        <assert test="count(addr:ADDRESS) = 1">
Validation error: <name/> element missing an ADDRESS child.
                        </assert>
                        <assert test="count(addr:EMAIL) = 1">
Validation error: <name/> element missing an EMAIL child.
                        </assert>
                        <assert test="addr:NAME[following-sibling::addr:ADDRESS] and addr:ADDRESS[following-sibling::addr:PHONENUM] and addr:PHONENUM[following-sibling::addr:EMAIL]">
Validation error: <name/> must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence
                        </assert>
                        <assert test="count(addr:NAME|addr:ADDRESS|addr:PHONENUM|addr:EMAIL) = count(*)">
Validation error: there is an extraneous element child of ENTRY
                        </assert>
                </rule>
                <rule context="addr:PHONENUM">
                        <assert test="@DESC">
Validation error: <name/> must have a DESC attribute
                        </assert>
                </rule>
        </pattern>
        <pattern name="Government Contact Report">
                <rule context="addr:EMAIL">
                        <report test="contains(., '.gov') and not(substring-after(., '.gov'))">
This address book contains government contacts.
                        </report>
                </rule>
        </pattern>
</schema>'''

#Listing 6 without root context hack
LISTING6_2 = '''\
<schema xmlns='http://purl.oclc.org/dsdl/schematron'>

        <ns prefix='addr' uri='http://addressbookns.com'/>

        <pattern name="Structural Validation">
                <rule context="/">
                        <assert test="addr:ADDRBOOK">Validation error: there must be an ADDRBOOK element at the root of the document.</assert>
                </rule>
                <rule context="addr:ENTRY">
                        <assert test="count(addr:*) + count(*[contains(namespace-uri(.), 'extension')]) = count(*)">
Validation error: all children of <name/> must either be in the namespace 'http://addressbookns.com' or in a namespace containing the substring 'extension'.
                        </assert>
                        <assert test="count(addr:NAME) = 1">
Validation error: <name/> element missing a NAME child.
                        </assert>
                        <assert test="count(addr:ADDRESS) = 1">
Validation error: <name/> element missing an ADDRESS child.
                        </assert>
                        <assert test="count(addr:EMAIL) = 1">
Validation error: <name/> element missing an EMAIL child.
                        </assert>
                        <assert test="addr:NAME[following-sibling::addr:ADDRESS] and addr:ADDRESS[following-sibling::addr:PHONENUM] and addr:PHONENUM[following-sibling::addr:EMAIL]">
Validation error: <name/> must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence
                        </assert>
                        <assert test="count(addr:NAME|addr:ADDRESS|addr:PHONENUM|addr:EMAIL) = count(*)">
Validation error: there is an extraneous element child of ENTRY
                        </assert>
                </rule>
                <rule context="addr:PHONENUM">
                        <assert test="@DESC">
Validation error: <name/> must have a DESC attribute
                        </assert>
                </rule>
        </pattern>
        <pattern name="Government Contact Report">
                <rule context="addr:EMAIL">
                        <report test="contains(., '.gov') and not(substring-after(., '.gov'))">
This address book contains government contacts.
                        </report>
                </rule>
        </pattern>
</schema>'''

LISTING2 = u'''\
<?xml version = "1.0"?>
<ADDRBOOK>
        <ENTRY ID="pa">
                <NAME>Pieter Aaron</NAME>
                <ADDRESS>404 Error Way</ADDRESS>
                <PHONENUM DESC="Work">404-555-1234</PHONENUM>
                <PHONENUM DESC="Fax">404-555-4321</PHONENUM>
                <PHONENUM DESC="Pager">404-555-5555</PHONENUM>
                <EMAIL>pieter.aaron@inter.net</EMAIL>
        </ENTRY>
        <ENTRY ID="en">
                <NAME>Emeka Ndubuisi</NAME>
                <ADDRESS>42 Spam Blvd</ADDRESS>
                <PHONENUM DESC="Work">767-555-7676</PHONENUM>
                <PHONENUM DESC="Fax">767-555-7642</PHONENUM>
                <PHONENUM DESC="Pager">800-SKY-PAGEx767676</PHONENUM>
                <EMAIL>endubuisi@spamtron.com</EMAIL>
        </ENTRY>
</ADDRBOOK>'''.encode(ENCODING)

LISTING4 = u'''\
<?xml version = "1.0"?>
<ADDRBOOK>
        <ENTRY ID="pa">
                <NAME>Pieter Aaron</NAME>
                <PHONENUM DESC="Work">404-555-1234</PHONENUM>
                <PHONENUM DESC="Fax">404-555-4321</PHONENUM>
                <PHONENUM DESC="Pager">404-555-5555</PHONENUM>
                <EMAIL>pieter.aaron@inter.net</EMAIL>
        </ENTRY>
        <ENTRY ID="en">
                <NAME>Emeka Ndubuisi</NAME>
                <PHONENUM DESC="Work">767-555-7676</PHONENUM>
                <PHONENUM DESC="Fax">767-555-7642</PHONENUM>
                <PHONENUM DESC="Pager">800-SKY-PAGEx767676</PHONENUM>
                <EMAIL>endubuisi@spamtron.com</EMAIL>
                <ADDRESS>42 Spam Blvd</ADDRESS>
                <SPAM>Make money fast</SPAM>
        </ENTRY>
        <EXTRA/>
</ADDRBOOK>
'''.encode(ENCODING)

LISTING7 = u'''\
<?xml version = "1.0"?>
<ADDRBOOK xmlns='http://addressbookns.com'>
        <ENTRY ID="pa">
                <NAME xmlns='http://bogus.com'>Pieter Aaron</NAME>
                <ADDRESS>404 Error Way</ADDRESS>
                <PHONENUM DESC="Work">404-555-1234</PHONENUM>
                <PHONENUM DESC="Fax">404-555-4321</PHONENUM>
                <PHONENUM DESC="Pager">404-555-5555</PHONENUM>
                <EMAIL>pieter.aaron@inter.net</EMAIL>
        </ENTRY>
        <ENTRY ID="en">
                <NAME xmlns='http://bogus.com'>Emeka Ndubuisi</NAME>
                <ADDRESS>42 Spam Blvd</ADDRESS>
                <PHONENUM DESC="Work">767-555-7676</PHONENUM>
                <PHONENUM DESC="Fax">767-555-7642</PHONENUM>
                <PHONENUM DESC="Pager">800-SKY-PAGEx767676</PHONENUM>
                <EMAIL>endubuisi@spamtron.com</EMAIL>
        </ENTRY>
</ADDRBOOK>
'''.encode(ENCODING)

LISTING8 = u'''\
<?xml version = "1.0"?>
<ADDRBOOK xmlns='http://addressbookns.com'>
        <ENTRY ID="pa">
                <NAME>Pieter Aaron</NAME>
                <ADDRESS>404 Error Way</ADDRESS>
                <PHONENUM DESC="Work">404-555-1234</PHONENUM>
                <PHONENUM DESC="Fax">404-555-4321</PHONENUM>
                <PHONENUM DESC="Pager">404-555-5555</PHONENUM>
                <EMAIL>pieter.aaron@inter.net</EMAIL>
        </ENTRY>
        <ENTRY ID="en">
                <NAME>Manfredo Manfredi</NAME>
                <ADDRESS>4414 Palazzo Terrace</ADDRESS>
                <PHONENUM DESC="Work">888-555-7676</PHONENUM>
                <PHONENUM DESC="Fax">888-555-7677</PHONENUM>
                <EMAIL>mpm@scudetto.dom.gov</EMAIL>
        </ENTRY>
</ADDRBOOK>'''.encode(ENCODING)


def normalize_text(text):
    #normalize text, a bit slow for large files with much non-ws
    return reduce(lambda a, b: (a and a[-1].isspace() and b.isspace()) and a or a+b, text, '')


class TestBasics(unittest.TestCase):
    def setUp(self):
        #self.isrc_factory = InputSource.DefaultFactory
        return

    def testStron1_1(self):
        output = run_stron(LISTING5_1, LISTING2)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Structural Validation\n'
        self.assertEqual(output, expected)
        return

    def testStron1_2(self):
        output = run_stron(LISTING5_2, LISTING2)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Structural Validation\n'
        self.assertEqual(output, expected)
        return

    def testStron1_3(self):
        output = run_stron(LISTING5_1, LISTING4)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Structural Validation\nAssertion failure:\nValidation error: ENTRY element missing an ADDRESS child.\nAssertion failure:\nValidation error: ENTRY must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence\nAssertion failure:\nValidation error: ENTRY must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence\nAssertion failure:\nValidation error: there is an extraneous element child of ENTRY\n'
        self.assertEqual(output, expected)
        return

    def testStron1_4(self):
        output = run_stron(LISTING5_2, LISTING4)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Structural Validation\nAssertion failure:\nValidation error: ENTRY element missing an ADDRESS child.\nAssertion failure:\nValidation error: ENTRY must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence\nAssertion failure:\nValidation error: ENTRY must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence\nAssertion failure:\nValidation error: there is an extraneous element child of ENTRY\n'
        self.assertEqual(output, expected)
        return

    def testStron2_1(self):
        output = run_stron(LISTING6_1, LISTING7)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Structural Validation\nAssertion failure:\nValidation error: all children of ENTRY must either be in the namespace \'http://addressbookns.com\' or in a namespace containing the substring \'extension\'.\nAssertion failure:\nValidation error: ENTRY element missing a NAME child.\nAssertion failure:\nValidation error: ENTRY must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence\nAssertion failure:\nValidation error: there is an extraneous element child of ENTRY\nAssertion failure:\nValidation error: all children of ENTRY must either be in the namespace \'http://addressbookns.com\' or in a namespace containing the substring \'extension\'.\nAssertion failure:\nValidation error: ENTRY element missing a NAME child.\nAssertion failure:\nValidation error: ENTRY must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence\nAssertion failure:\nValidation error: there is an extraneous element child of ENTRY\nProcessing pattern: Government Contact Report\n'
        self.assertEqual(output, expected)
        return

    def testStron2_2(self):
        output = run_stron(LISTING6_2, LISTING7)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Structural Validation\nAssertion failure:\nValidation error: all children of ENTRY must either be in the namespace \'http://addressbookns.com\' or in a namespace containing the substring \'extension\'.\nAssertion failure:\nValidation error: ENTRY element missing a NAME child.\nAssertion failure:\nValidation error: ENTRY must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence\nAssertion failure:\nValidation error: there is an extraneous element child of ENTRY\nAssertion failure:\nValidation error: all children of ENTRY must either be in the namespace \'http://addressbookns.com\' or in a namespace containing the substring \'extension\'.\nAssertion failure:\nValidation error: ENTRY element missing a NAME child.\nAssertion failure:\nValidation error: ENTRY must have a NAME, ADDRESS, one or more PHONENUM, and an EMAIL in sequence\nAssertion failure:\nValidation error: there is an extraneous element child of ENTRY\nProcessing pattern: Government Contact Report\n'
        self.assertEqual(output, expected)
        return

    def testStron2_3(self):
        output = run_stron(LISTING6_1, LISTING8)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Structural Validation\nProcessing pattern: Government Contact Report\nReport:\nThis address book contains government contacts.\n'
        self.assertEqual(output, expected)
        return

    def testStron2_4(self):
        output = run_stron(LISTING6_2, LISTING8)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Structural Validation\nProcessing pattern: Government Contact Report\nReport:\nThis address book contains government contacts.\n'
        self.assertEqual(output, expected)
        return


if __name__ == '__main__':
    unittest.main()




#Test examples in http://uche.ogbuji.net/tech/pubs/schematron.html
import sys
import unittest
from util import run_stron

ABS_RULE_STRON = '''\
<sch:schema xmlns:sch="http://www.ascc.net/xml/schematron" >
     <sch:title>Abstract rules</sch:title>
     <sch:pattern name="Check items" see="http://zvon.org/friendsRule.html">
          <sch:rule context="email">
               <sch:extends rule="phone"/>
               <sch:extends rule="email"/>
          </sch:rule>
          <sch:rule abstract="yes" id="phone">
               <sch:assert test="../phone">There is no phone.</sch:assert>
          </sch:rule>
          <sch:rule abstract="yes" id="email">
               <sch:assert test="contains(.,'@')"> @ is missing</sch:assert>
               <sch:report test="contains(normalize-space(.),' ')">Whitespace inside address</sch:report>
          </sch:rule>
     </sch:pattern>
</sch:schema>'''

ABS_RULE_CANDIDATE = '''\
<friends>
     <friend>
          <name>John Smith</name>
          <email>John.Smith#yahoo.com</email>
          <phone>123452</phone>
     </friend>
     <friend>
          <name>Jane Brown</name>
          <email>Jane Brown@home.org</email>
     </friend>
</friends> '''


def normalize_text(text):
    #normalize text, a bit slow for large files with much non-ws
    return reduce(lambda a, b: (a and a[-1].isspace() and b.isspace()) and a or a+b, text, '')


class TestZvon(unittest.TestCase):
    def testAbsRule(self):
        output = run_stron(ABS_RULE_STRON, ABS_RULE_CANDIDATE, True)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Abstract rules\nProcessing pattern: Check items\nAssertion failure:\n@ is missing\nAssertion failure:\nThere is no phone.\nReport:\nWhitespace inside address\n'
        self.assertEqual(output, expected)
        return


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
"""
A command line tool for running reports on XML files.

trimxslt allows you to rapidly extract details from large XML files
on the command line.

Run "trimxslt --help" for details of the command line parameters, but
here are some pointers to get you started.

Let's say you have a simple database dump format with the following
form:

<db>
  <record id="1">
    <name>Alex</name>
    <address>123 Maple St.</address>
  </record>
  <record id="2">
    <name>Bob</name>
    <address>456 Birch Rd.</address>
  </record>
  <record id="3">
    <name>Chris</name>
    <address>789 Pine St.</address>
  </record>
</db>

You can:

Get all the full contents of name elements

$ trimxml file.xml name
<name>Alex</name>
<name>Bob</name>
<name>Chris</name>

Get the full contents of the record with ID 2

$ trimxml file.xml record "@id='2'"
<record id="2">
    <name>Bob</name>
    <address>456 Birch Rd.</address>
  </record>

Get the full contents of the first two name elements

$ trimxml -c 2 file.xml name
<name>Alex</name>
<name>Bob</name>

Get the name of the record with ID 2

$ trimxml -d "name" file.xml record "@id='2'"
<name>Bob</name>

You could display the id and each correspoding name as follows:

$ trimxml file.xml "@id|name"
1
<name>Alex</name>
2
<name>Bob</name>
3
<name>Chris</name>

Or a more precise approach might be (demonstrating the use of XPath functions):

$ trimxml -d "concat(@id, ': ', name)" file.xml record
1: Alex
2: Bob
3: Chris

trimxml uses namespaces declared on the document element, so you can
conveniently make queries without needing to separately declare prefixes.
So to get the URLs of all a links in an XHTML document you could do:

trimxml -d "@href" file.xhtml "html:a"

As long as there is a namespace declaration
xmlns:ht="http://www.w3.org/1999/xhtml" in the document.  If not
(many XHTML documents use the default namespace, which courtesy XPath 1.0
restrictions prevents trimxml from doing any guesswork for you) you have
to declare the prefix.

trimxml --ns=ht="http://www.w3.org/1999/xhtml" -d "@href" http://www.w3.org/2000/07/8378/xhtml/media-types/test4.xhtml "ht:a"

Notice how this example loads the source XML (XHTML) from a Web URL rather than a local file.  Of course, a shortcut for this is simply:

trimxml http://www.w3.org/2000/07/8378/xhtml/media-types/test4.xhtml "@href"

"""
#The following won't work because EXSLT is only supported in XsltContext and we use Ft.Xml.XPath.Context
#We can probably revisit when we make bindery nodes subclasses of Domlette
#trimxml --ns=str="http://exslt.org/strings" -d "str:replace(., 'http://', '')" http://www.w3.org/2000/07/8378/xhtml/media-types/test4.xhtml "@href"

import os
import re
import sys
import codecs
import optparse
#import cStringIO
import amara
from amara import saxtools
from xml.dom import Node

#from xml.dom import EMPTY_NAMESPACE as NULL_NAMESPACE
#from xml.dom import EMPTY_PREFIX as NULL_PREFIX


#FIXME: Use 4Suite L10N
def _(t): return t


def run(source, xpattern, xpath, limit, sentinel, display, prefixes):
    prefixes = prefixes or {}
    try:
        prefixes = dict([ p.split('=') for p in prefixes ])
    except ValueError:
        raise ValueError("Invalid prefix declaration")
    if hasattr(source, 'read'):
        if hasattr(source, 'rewind'):
            nss = saxtools.sniff_namespace(source)
            source.rewind()
        else:
            source = source.read()
            nss = saxtools.sniff_namespace(source)
    else:
        nss = saxtools.sniff_namespace(source)
    nss.update(prefixes)
    nodes = amara.pushbind(source, xpattern, prefixes=nss)
    count = 0
    for node in nodes:
        if not xpath or node.xml_xpath(xpath):
            count += 1
            if display:
                #Print specified subset
                result = node.xml_xpath(display)
                if isinstance(result, list):
                    print '\n'.join([ n.nodeType == Node.ATTRIBUTE_NODE and n.nodeValue or n.xml() for n in result ])
                else:
                    print result
            else:
                #Print the whole thing
                try:
                    print node.xml()
                except AttributeError:
                    print unicode(node).encode('utf-8')
            if limit != -1 and count >= limit:
                break
        if sentinel and node.xml_xpath(sentinel):
            break
    return


class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg


def command_line_prep():
    from optparse import OptionParser
    usage = "%prog [options] source xpattern [xpath]"
    parser = OptionParser(usage=usage)
    parser.add_option("-c", "--limit",
                      action="store", type="int", dest="limit", default=-1,
                      help="limit the number of xpattern matches retrieved; files will not be parsed beyond this number, so it serves as optimization", metavar="NUMBER")
    parser.add_option("-d", "--display",
                      action="store", type="string", dest="display",
                      help="xpath expression indicating what nodes to be displayed from matched and screened patterns", metavar="XPATH")
    parser.add_option("-n", "--ns",
                      action="append", type="string", dest="ns",
                      help="prefix to namespace mapping", metavar="<PREFIX=URI>")
    parser.add_option("--sentinel",
                      action="store", type="string", dest="sentinel",
                      help="xpath expression to be checked for each pattern match.  If true it causes the   reporting to stop, with no further parsing", metavar="XPATH")
    #parser.add_option("-q", "--quiet",
    #                  action="store_false", dest="verbose", default=1,
    #                  help="don't print status messages to stdout")
    return parser


def main(argv=None):
    #Ideas borrowed from
    # http://www.artima.com/forums/flat.jsp?forum=106&thread=4829
    #But with better integration of entry points
    if argv is None:
        argv = sys.argv
    # By default, optparse usage errors are terminated by SystemExit
    try:
        optparser = command_line_prep()
        options, args = optparser.parse_args(argv[1:])
        # Process mandatory arguments with IndexError try...except blocks
        try:
            source = args[0]
        except IndexError:
            optparser.error("Missing filename/URL to parse")
        try:
            xpattern = args[1]
        except IndexError:
            optparser.error("Missing main xpattern")
    except SystemExit, status:
        return status

    # Perform additional setup work here before dispatching to run()
    # Detectable errors encountered here should be handled and a status
    # code of 1 should be returned. Note, this would be the default code
    # for a SystemExit exception with a string message.
    try:
        xpath = args[2].decode('utf-8')
    except IndexError:
        xpath = None
    xpattern = xpattern.decode('utf-8')
    sentinel = options.sentinel and options.sentinel.decode('utf-8')
    display = options.display and options.display.decode('utf-8')
    prefixes = options.ns
    limit = options.limit
    if source == '-':
        source = sys.stdin
    run(source, xpattern, xpath, limit, sentinel, display, prefixes)


if __name__ == "__main__":
    sys.exit(main(sys.argv))

